@@ -22,7 +22,7 @@
 
 #include <freetds/tds.h>
 #include <freetds/convert.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 
 /*
  * Internal (not part of the exposed API) prototypes and such.
@@ -98,6 +98,8 @@ struct _cs_context
 	/* code changes end here - CS_CONFIG - 01*/
 	TDSCONTEXT *tds_ctx;
 	CS_CONFIG config;
+	int login_timeout;  /**< not used unless positive */
+	int query_timeout;  /**< not used unless positive */
 };
 
 /*
