@@ -17,9 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdio.h>
 #include <assert.h>
@@ -49,13 +47,14 @@
 #include <locale.h>
 #endif
 
+#include <freetds/windows.h>
+
 #include "tds_sysdep_public.h"
 #include <sql.h>
 #include <sqlext.h>
 #include "replacements.h"
 
-static char software_version[] = "$Id: bsqlodbc.c,v 1.17 2010/07/22 09:55:37 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#include <freetds/sysdep_private.h>
 
 static char * next_query(void);
 static void print_results(SQLHSTMT hStmt);
@@ -383,7 +382,7 @@ next_query()
 		/* Add the query line to the command to be sent to the server */
 		if (!strlen(query_line))
 			continue;
-		p = realloc(sql, 1 + (sql? strlen(sql) : 0) + strlen(query_line));
+		p = (char *) realloc(sql, 1 + (sql? strlen(sql) : 0) + strlen(query_line));
 		if (!p) {
 			fprintf(stderr, "%s:%d: could not allocate memory\n", options.appname, __LINE__);
 			return NULL;
@@ -548,16 +547,17 @@ print_results(SQLHSTMT hStmt)
 			assert(namelen < sizeof(name));
 			name[namelen] = '\0';
 			metadata[c].name = strdup((char *) name);
-			metadata[c].width = (ndigits > metadata[c].size)? ndigits : metadata[c].size;
+			metadata[c].width =
+				(ndigits >= 0 && (unsigned) ndigits > metadata[c].size)? ndigits : metadata[c].size;
 			
 			if (is_character_data(metadata[c].type)) {
 				SQLHDESC hDesc;
 				SQLINTEGER buflen;
-				
+
 				metadata[c].nchars = metadata[c].size;
-				
-				if ((erc = SQLAllocHandle(SQL_HANDLE_DESC, hStmt, &hDesc)) != SQL_SUCCESS) {
-					odbc_perror(hStmt, erc, "SQLAllocHandle", "failed");
+
+				if ((erc = SQLGetStmtAttr(hStmt, SQL_ATTR_IMP_ROW_DESC, &hDesc, sizeof(hDesc), NULL)) != SQL_SUCCESS) {
+					odbc_perror(hStmt, erc, "SQLGetStmtAttr", "failed");
 					exit(EXIT_FAILURE);
 				} 
 				if ((erc = SQLGetDescField(hDesc, c+1, SQL_DESC_OCTET_LENGTH, 
@@ -565,12 +565,8 @@ print_results(SQLHSTMT hStmt)
 								&buflen)) != SQL_SUCCESS) {
 					odbc_perror(hStmt, erc, "SQLGetDescField", "failed");
 					exit(EXIT_FAILURE);
-				} 
-				
-				if ((erc = SQLFreeHandle(SQL_HANDLE_DESC, hStmt)) != SQL_SUCCESS) {
-					odbc_perror(hStmt, erc, "SQLFreeHandle", "failed");
-					exit(EXIT_FAILURE);
-				} 
+				}
+				metadata[c].size = metadata[c].size * 2 + 1;
 			}
 
 			fprintf(options.verbose, "%6d  %30s  %10d  %18s  %6lu  %6d  \n", 
@@ -593,6 +589,7 @@ print_results(SQLHSTMT hStmt)
 			ret = set_format_string(&metadata[c], (c+1 < ncols)? options.colsep : "\n");
 			if (ret <= 0) {
 				fprintf(stderr, "%s:%d: asprintf(), column %d failed\n", options.appname, __LINE__, c+1);
+				free_metadata(metadata, data, ncols);
 				return;
 			}
 
@@ -607,7 +604,7 @@ print_results(SQLHSTMT hStmt)
 			 * inaccesible to the application.  
 			 */
 
-			data[c].buffer = calloc(1, bufsize(&metadata[c]));
+			data[c].buffer = (char *) calloc(1, bufsize(&metadata[c]));
 			assert(data[c].buffer);
 
 			if ((erc = SQLBindCol(hStmt, c+1, SQL_C_CHAR, (SQLPOINTER)data[c].buffer, 
@@ -643,18 +640,14 @@ print_results(SQLHSTMT hStmt)
 				exit(EXIT_FAILURE);
 			}
 			for (c=0; c < ncols; c++) {
-				char *s;
 				switch (data[c].len) { /* handle nulls */
 				case SQL_NULL_DATA: /* is null */
 					fprintf(stdout, metadata[c].format_string, "NULL");
 					break;
 				default:
-					assert(data[c].len > 0);
-					s = calloc(1, 1 + data[c].len);
-					assert(s);
-					memcpy(s, data[c].buffer, data[c].len);
-					fprintf(stdout, metadata[c].format_string, s);
-					free(s);
+					assert(data[c].len >= 0);
+				case SQL_NO_TOTAL:
+					fprintf(stdout, metadata[c].format_string, data[c].buffer);
 					break;
 				}
 			}
@@ -682,60 +675,9 @@ print_results(SQLHSTMT hStmt)
 		assert(erc != SQL_STILL_EXECUTING);
 		odbc_perror(hStmt, erc, "SQLMoreResults", "failed");
 		exit(EXIT_FAILURE);
-	} 
-}
-
-#if 0 
-static int
-get_printable_size(int type, int size)	/* adapted from src/dblib/dblib.c */
-{
-	switch (type) {
-	case SYBINTN:
-		switch (size) {
-		case 1:
-			return 3;
-		case 2:
-			return 6;
-		case 4:
-			return 11;
-		case 8:
-			return 21;
-		}
-	case SYBINT1:
-		return 3;
-	case SYBINT2:
-		return 6;
-	case SYBINT4:
-		return 11;
-	case SYBINT8:
-		return 21;
-	case SYBVARCHAR:
-	case SYBCHAR:
-		return size;
-	case SYBFLT8:
-		return 11;	/* FIX ME -- we do not track precision */
-	case SYBREAL:
-		return 11;	/* FIX ME -- we do not track precision */
-	case SYBMONEY:
-		return 12;	/* FIX ME */
-	case SYBMONEY4:
-		return 12;	/* FIX ME */
-	case SYBDATETIME:
-		return 26;	/* FIX ME */
-	case SYBDATETIME4:
-		return 26;	/* FIX ME */
-#if 0	/* seems not to be exported to sybdb.h */
-	case SYBBITN:
-#endif
-	case SYBBIT:
-		return 1;
-		/* FIX ME -- not all types present */
-	default:
-		return 0;
 	}
-
+	free_metadata(metadata, data, ncols);
 }
-#endif /* 0, not used */
 
 /** 
  * Build the column header format string, based on the column width. 
@@ -744,13 +686,14 @@ get_printable_size(int type, int size)	/* adapted from src/dblib/dblib.c */
 static int
 set_format_string(struct METADATA * meta, const char separator[])
 {
-	int width, ret;
+	int ret;
+	unsigned width;
 	const char *size_and_width;
 	assert(meta);
 
 	if(0 == strcmp(options.colsep, default_colsep)) { 
 		/* right justify numbers, left justify strings */
-		size_and_width = is_character_data(meta->type)? "%%-%d.%ds%s" : "%%%d.%ds%s";
+		size_and_width = is_character_data(meta->type)? "%%-%u.%us%s" : "%%%u.%us%s";
 		
 		width = meta->width; /* get_printable_size(meta->type, meta->size); */
 		if (width < strlen(meta->name))
@@ -823,11 +766,11 @@ get_login(int argc, char *argv[], OPTIONS *options)
 
 	assert(options && argv);
 	
-	options->appname = tds_basename(argv[0]);
+	options->appname = basename(argv[0]);
 	options->colsep = default_colsep; /* may be overridden by -t */
 	options->odbc_version = SQL_OV_ODBC3;	  /* may be overridden by -V */
 	
-	login = calloc(1, sizeof(LOGINREC));
+	login = (LOGINREC *) calloc(1, sizeof(LOGINREC));
 	
 	if (!login) {
 		fprintf(stderr, "%s: unable to allocate login structure\n", options->appname);
@@ -838,7 +781,7 @@ get_login(int argc, char *argv[], OPTIONS *options)
 		perror("unable to get hostname");
 	} 
 
-	while ((ch = getopt(argc, argv, "U:P:S:dD:i:o:e:t:V:hqv")) != -1) {
+	while ((ch = getopt(argc, argv, "U:P:S:d:D:i:o:e:t:V:hqv")) != -1) {
 		char *p;
 		switch (ch) {
 		case 'U':
