@@ -42,6 +42,8 @@
 #include <sys/stat.h>
 #endif /* HAVE_SYS_STAT_H */
 
+#include <freetds/bool.h>
+
 #include <sybfront.h>
 #include <sybdb.h>
 #include "terminal.h"
@@ -212,6 +214,252 @@ get_printable_column_size(DBPROCESS * dbproc, int col)
 	return collen;
 }
 
+static bool
+update_comment(bool in_comment, const char *line)
+{
+	const char *p = line;
+	while (*p) {
+		if (in_comment) {
+			p = strstr(p, "*/");
+			if (!p)
+				break;
+			p += 2;
+			in_comment = false;
+		}
+		/* not comment */
+		if (p[0] == '/' && p[1] == '*') {
+			in_comment = true;
+			p += 2;
+		} else if (p[0] == '-' && p[1] == '-') {
+			/* comment till the end of line then no comment
+			 * on next line */
+			break;
+		} else if (p[0]) {
+			++p;
+		}
+	}
+	return in_comment;
+}
+
+static DBPROCESS *dbproc;
+static const char *editor;
+static char **ibuf = NULL;
+static int ibuflines = 0;
+static int no_prompt = 0;
+static int default_exit = EXIT_SUCCESS;
+static const char *cmdend = "go";
+
+static void
+reset_ibuf(void)
+{
+	int i;
+	for (i = 0; ibuf && i < ibuflines; i++) {
+		free(ibuf[i]);
+	}
+	ibuflines = 0;
+	free(ibuf);
+	ibuf = NULL;
+}
+
+static void
+append_line(char *line)
+{
+	ibuf = (char **) xrealloc(ibuf, (ibuflines + 1) * sizeof(char *));
+	ibuf[ibuflines++] = line;
+}
+
+static void
+vi_cmd(const char *command)
+{
+	char tmpfn[256];
+	FILE *fp;
+	int i;
+	char *sqlch;
+	FILE *tmpfp;
+	char *line;
+	char foobuf[40];
+	int tmpfd;
+	mode_t old_mask;
+
+	strcpy(tmpfn, "/tmp/fisqlXXXXXX");
+	old_mask = umask(077);
+	tmpfd = mkstemp(tmpfn);
+	umask(old_mask);
+	if (tmpfd < 0 || (fp = fdopen(tmpfd, "w")) == NULL) {
+		perror("fisql");
+		reset_term();
+		dbexit();
+		exit(2);
+	}
+	if (ibuflines) {
+		for (i = 0; i < ibuflines; i++) {
+			fputs(ibuf[i], fp);
+			fputc('\n', fp);
+			free(ibuf[i]);
+		}
+	} else {
+		for (i = 0; ((sqlch = dbgetchar(dbproc, i)) != NULL); i++) {
+			fputc(*sqlch, fp);
+		}
+	}
+	fclose(fp);
+
+	edit(command, tmpfn);
+
+	ibuflines = 0;
+	fp = fopen(tmpfn, "r");
+	if (!fp) {
+		perror("fisql");
+		reset_term();
+		dbexit();
+		exit(2);
+	}
+	tmpfp = rl_instream;
+	rl_instream = fp;
+	strcpy(foobuf, "1>> ");
+	while ((line = readline(foobuf)) != NULL) {
+		append_line(line);
+		sprintf(foobuf, "%d>> ", ibuflines + 1);
+	}
+	rl_instream = tmpfp;
+	fclose(fp);
+	fputc('\r', stdout);
+	fflush(stdout);
+	unlink(tmpfn);
+}
+
+static void
+system_cmd(const char *cmd)
+{
+	int rv;
+	switch (rv = system(cmd)) {
+	case 0:
+		break;
+	case -1:
+		fprintf(stderr,
+		    "Failed to execute `%s'\n", cmd);
+		break;
+	default:
+		fprintf(stderr, "Command `%s' exited "
+		    "with code %d\n", cmd, rv);
+		break;
+	}
+}
+
+static void
+readfile_cmd(char *line)
+{
+	FILE *fp;
+	FILE *tmpfp;
+	FILE *tmpfp2;
+	char *tfn;
+	char *cp;
+
+	for (cp = line + 2; *cp && (isspace((unsigned char) *cp)); cp++)
+		continue;
+	tfn = cp;
+	for (; *cp && !(isspace((unsigned char) *cp)); cp++)
+		continue;
+	*cp = '\0';
+	if ((fp = fopen(tfn, "r")) == NULL) {
+		fprintf(stderr, "Operating system error: Failed to open %s.\n", tfn);
+		return;
+	}
+	tmpfp = rl_instream;
+	tmpfp2 = rl_outstream;
+	rl_instream = fp;
+	rl_outstream = fopen("/dev/null", "w");
+	while ((line = readline("")) != NULL) {
+		append_line(line);
+	}
+	rl_instream = tmpfp;
+	fclose(rl_outstream);
+	rl_outstream = tmpfp2;
+	fclose(fp);
+	fputc('\r', stdout);
+	fflush(stdout);
+}
+
+static void
+read_sql_lines(void)
+{
+	char *cp;
+	char *firstword;
+	char firstword_separator;
+	char *line = NULL;
+	char foobuf[40];
+	bool in_comment = false;
+
+	reset_ibuf();
+	while (1) {
+		if (no_prompt) {
+			foobuf[0] = '\0';
+		} else {
+			sprintf(foobuf, "%d>> ", ibuflines + 1);
+		}
+		free(line);
+		line = readline(foobuf);
+		if (line == NULL) {
+			reset_term();
+			dbexit();
+			exit(default_exit);
+		}
+		for (cp = line; *cp && isspace((unsigned char) *cp); cp++)
+			continue;
+		if (*cp) {
+			add_history(line);
+		}
+		if (in_comment) {
+			goto append_line;
+		}
+		if (!(strncasecmp(line, "!!", 2))) {
+			system_cmd(line + 2);
+			continue;
+		}
+		/* XXX: isql off-by-one line count error for :r not duplicated */
+		if (!(strncasecmp(line, ":r", 2))) {
+			readfile_cmd(line);
+			continue;
+		}
+		firstword = line;
+		firstword_separator = '\0';
+		for (cp = firstword; *cp; cp++) {
+			if (isspace((unsigned char) *cp)) {
+				firstword_separator = *cp;
+				*cp = '\0';
+				break;
+			}
+		}
+		if ((!(strcasecmp(firstword, "exit")))
+		    || (!(strcasecmp(firstword, "quit")))) {
+			reset_term();
+			dbexit();
+			exit(default_exit);
+		}
+		if (!(strcasecmp(firstword, "reset"))) {
+			reset_ibuf();
+			continue;
+		}
+		if (!(strcasecmp(firstword, cmdend))) {
+			if (ibuflines == 0) {
+				continue;
+			}
+			break;
+		}
+		if ((!(strcasecmp(firstword, "vi")))
+		    || (!(strcasecmp(firstword, editor)))) {
+			vi_cmd(firstword);
+			continue;
+		}
+		firstword[strlen(firstword)] = firstword_separator;
+
+append_line:
+		in_comment = update_comment(in_comment, line);
+		append_line(line);
+		line = NULL;
+	}
+}
+
 int
 main(int argc, char *argv[])
 {
@@ -222,10 +470,8 @@ main(int argc, char *argv[])
 #endif
 	int fipsflagger = 0;
 	int perfstats = 0;
-	int no_prompt = 0;
 	int use_encryption = 0;
 	int chained_transactions = 0;
-	const char *cmdend = "go";
 	int headers = 0;
 	char *columnwidth = NULL;
 	const char *colseparator = " ";
@@ -235,9 +481,7 @@ main(int argc, char *argv[])
 	char *password = NULL;
 	char *server = NULL;
 	DBCHAR *char_set = NULL;
-	const char *editor;
 	char *hostname = NULL;
-	char *sqlch;
 	char *interfaces_filename = NULL;
 	char *input_filename = NULL;
 	char *output_filename = NULL;
@@ -245,26 +489,14 @@ main(int argc, char *argv[])
 	char *language = NULL;
 	int size = 0;
 	char *sybenv;
-	DBPROCESS *dbproc;
 	LOGINREC *login;
-	char **ibuf = NULL;
-	int ibuflines = 0;
 	int printedlines;
 	int i;
-	char *line;
 	int dbrc;
-	char foobuf[512];
-	char *firstword;
-	char *cp;
 	int c;
 	int errflg = 0;
 	char *prbuf;
 	int prbuflen;
-	FILE *fp;
-	FILE *tmpfp;
-	FILE *tmpfp2;
-	char *tfn;
-	char tmpfn[256];
 	int num_cols;
 	int selcol;
 	int col;
@@ -276,7 +508,6 @@ main(int argc, char *argv[])
 	int printedcompute = 0;
 	char adash;
 	const char *database_name = NULL;
-	int default_exit = EXIT_SUCCESS;
 
 	setlocale(LC_ALL, "");
 
@@ -316,7 +547,7 @@ main(int argc, char *argv[])
 			break;
 		case 'v':
 			puts("fisql, a free isql replacement by Nicholas S. Castellano");
-			exit(EXIT_SUCCESS);
+			return EXIT_SUCCESS;
 			break;
 		case 'X':
 			/* XXX: We get a different error message than isql gives; neither seems
@@ -409,7 +640,7 @@ main(int argc, char *argv[])
 		fprintf(stderr, "\t[-P password] [-s colseparator] [-S server]\n");
 		fprintf(stderr, "\t[-t timeout] [-U username] [-w columnwidth]\n");
 		fprintf(stderr, "\t[-y sybase_dir] [-z language]\n");
-		exit(EXIT_FAILURE);
+		return EXIT_FAILURE;
 	}
 	if (!(isatty(fileno(stdin)))) {
 		no_prompt = 1;
@@ -428,7 +659,7 @@ main(int argc, char *argv[])
 			 */
 			/* lack of newline for bug-compatibility with isql */
 			fprintf(stderr, "Unable to open input file '%s'.", optarg);
-			exit(EXIT_FAILURE);
+			return EXIT_FAILURE;
 		}
 	}
 	if (output_filename) {
@@ -438,7 +669,7 @@ main(int argc, char *argv[])
 			 */
 			/* lack of newline for bug-compatibility with isql */
 			fprintf(stderr, "Unable to open output file '%s'.", output_filename);
-			exit(EXIT_FAILURE);
+			return EXIT_FAILURE;
 		}
 	}
 	if (isatty(fileno(stdin))) {
@@ -452,7 +683,7 @@ main(int argc, char *argv[])
 #endif
 	if ((login = dblogin()) == NULL) {
 		reset_term();
-		exit(EXIT_FAILURE);
+		return EXIT_FAILURE;
 	}
 	dbmsghandle(msg_handler);
 	dberrhandle(err_handler);
@@ -491,7 +722,7 @@ main(int argc, char *argv[])
 	if ((dbproc = dbopen(login, server)) == NULL) {
 		fprintf(stderr, "fisql: dbopen() failed.\n");
 		reset_term();
-		exit(EXIT_FAILURE);
+		return EXIT_FAILURE;
 	}
 
 	dbsetopt(dbproc, DBPRLINESEP, lineseparator, strlen(lineseparator));
@@ -513,162 +744,16 @@ main(int argc, char *argv[])
 
 	while (1) {
 		if (sigsetjmp(restart, 1)) {
-			if (ibuf) {
-				for (i = 0; i < ibuflines; i++) {
-					free(ibuf[i]);
-				}
-				ibuflines = 0;
-				free(ibuf);
-				ibuf = NULL;
-			}
+			reset_ibuf();
 			fputc('\n', stdout);
 			rl_on_new_line();
 			rl_reset_line_state();
 		}
 		dbcancel(dbproc);
 		signal(SIGINT, inactive_interrupt_handler);
-		ibuf = (char **) xmalloc(sizeof(char *));
-		ibuflines = 0;
-		while (1) {
-			if (no_prompt) {
-				foobuf[0] = '\0';
-			} else {
-				sprintf(foobuf, "%d>> ", ibuflines + 1);
-			}
-			line = readline(foobuf);
-			if (line == NULL) {
-				line = "exit";
-			}
-			for (cp = line; *cp && isspace((unsigned char) *cp); cp++)
-				continue;
-			if (*cp) {
-				add_history(line);
-			}
-			if (!(strncasecmp(line, "!!", 2))) {
-				int rv;
-				cp = line + 2;
-				switch (rv = system(cp)) {
-				case 0:
-					continue;
-				case -1:
-					fprintf(stderr,
-					    "Failed to execute `%s'\n", cp);
-					continue;
-				default:
-					fprintf(stderr, "Command `%s' exited "
-					    "with code %d\n", cp, rv);
-					continue;
-				}
-			}
-			/* XXX: isql off-by-one line count error for :r not duplicated */
-			if (!(strncasecmp(line, ":r", 2))) {
-				for (cp = line + 2; *cp && (isspace((unsigned char) *cp)); cp++)
-					continue;
-				tfn = cp;
-				for (; *cp && !(isspace((unsigned char) *cp)); cp++)
-					continue;
-				*cp = '\0';
-				if ((fp = fopen(tfn, "r")) == NULL) {
-					fprintf(stderr, "Operating system error: Failed to open %s.\n", tfn);
-					continue;
-				}
-				tmpfp = rl_instream;
-				tmpfp2 = rl_outstream;
-				rl_instream = fp;
-				rl_outstream = fopen("/dev/null", "w");
-				while ((line = readline("")) != NULL) {
-					ibuf[ibuflines++] = line;
-					ibuf = (char **) xrealloc(ibuf, (ibuflines + 1) * sizeof(char *));
-				}
-				rl_instream = tmpfp;
-				fclose(rl_outstream);
-				rl_outstream = tmpfp2;
-				fclose(fp);
-				fputc('\r', stdout);
-				fflush(stdout);
-				continue;
-			}
-			firstword = (char *) xmalloc((strlen(line) + 1) * sizeof(char));
-			strcpy(firstword, line);
-			for (cp = firstword; *cp; cp++) {
-				if (isspace((unsigned char) *cp)) {
-					*cp = '\0';
-					break;
-				}
-			}
-			if ((!(strcasecmp(firstword, "exit")))
-			    || (!(strcasecmp(firstword, "quit")))) {
-				reset_term();
-				dbexit();
-				exit(default_exit);
-			}
-			if (!(strcasecmp(firstword, "reset"))) {
-				for (i = 0; i < ibuflines; i++) {
-					free(ibuf[i]);
-				}
-				ibuflines = 0;
-				continue;
-			}
-			if (!(strcasecmp(firstword, cmdend))) {
-				if (ibuflines == 0) {
-					continue;
-				}
-				free(firstword);
-				break;
-			}
-			if ((!(strcasecmp(firstword, "vi")))
-			    || (!(strcasecmp(firstword, editor)))) {
-				int tmpfd;
-				mode_t old_mask;
 
-				strcpy(tmpfn, "/tmp/fisqlXXXXXX");
-				old_mask = umask(077);
-				tmpfd = mkstemp(tmpfn);
-				umask(old_mask);
-				if ((fp = fdopen(tmpfd, "w")) == NULL) {
-					perror("fisql");
-					reset_term();
-					dbexit();
-					exit(2);
-				}
-				if (ibuflines) {
-					for (i = 0; i < ibuflines; i++) {
-						fputs(ibuf[i], fp);
-						fputc('\n', fp);
-						free(ibuf[i]);
-					}
-				} else {
-					for (i = 0; ((sqlch = dbgetchar(dbproc, i)) != NULL); i++) {
-						fputc(*sqlch, fp);
-					}
-				}
-				fclose(fp);
-				if (!(strcmp(firstword, "vi"))) {
-					edit("vi", tmpfn);
-				} else {
-					edit(editor, tmpfn);
-				}
-				ibuflines = 0;
-				fp = fopen(tmpfn, "r");
-				tmpfp = rl_instream;
-				rl_instream = fp;
-				strcpy(foobuf, "1>> ");
-				while ((line = readline(foobuf)) != NULL) {
-					ibuf[ibuflines++] = line;
-					sprintf(foobuf, "%d>> ", ibuflines + 1);
-					ibuf = (char **) xrealloc(ibuf, (ibuflines + 1) * sizeof(char *));
-				}
-				rl_instream = tmpfp;
-				fclose(fp);
-				fputc('\r', stdout);
-				fflush(stdout);
-				unlink(tmpfn);
-				continue;
-			}
-			free(firstword);
-			ibuf[ibuflines++] = line;
-			ibuf = (char **) xrealloc(ibuf, (ibuflines + 1) * sizeof(char *));
-		}
+		read_sql_lines();
+
 		dbfreebuf(dbproc);
 		for (i = 0; i < ibuflines; i++) {
 			if (echo) {
@@ -683,6 +768,8 @@ main(int argc, char *argv[])
 		signal(SIGINT, active_interrupt_handler);
 		dbsetinterrupt(dbproc, (void *) active_interrupt_pending, (void *) active_interrupt_servhandler);
 		if (dbsqlexec(dbproc) == SUCCEED) {
+			int status_printed = 0;
+
 			maybe_handle_active_interrupt();
 			while ((dbrc = dbresults(dbproc)) != NO_MORE_RESULTS) {
 				printedlines = 0;
@@ -802,12 +889,14 @@ main(int argc, char *argv[])
 						if (DBCOUNT(dbproc) >= 0) {
 							fprintf(stdout, "(%d rows affected", (int) DBCOUNT(dbproc));
 							if (dbhasretstat(dbproc)) {
+								status_printed = 1;
 								dbrc = dbretstatus(dbproc);
 								fprintf(stdout, ", return status = %d", dbrc);
 							}
 							fprintf(stdout, ")\n");
 						} else {
 							if (dbhasretstat(dbproc)) {
+								status_printed = 1;
 								dbrc = dbretstatus(dbproc);
 								fprintf(stdout, "(return status = %d)\n", dbrc);
 							}
@@ -815,6 +904,10 @@ main(int argc, char *argv[])
 					}
 				}
 			}
+			if (!status_printed && dbhasretstat(dbproc)) {
+				dbrc = dbretstatus(dbproc);
+				fprintf(stdout, "(return status = %d)\n", dbrc);
+			}
 		} else {
 			/* Something failed, so change the default
 			 * exit status to reflect that.
@@ -824,6 +917,5 @@ main(int argc, char *argv[])
 	}
 	reset_term();
 	dbexit();
-	exit(EXIT_FAILURE);
-	return (0);
+	return EXIT_FAILURE;
 }
