@@ -78,9 +78,10 @@
 
 #include <freetds/tds.h>
 #include <freetds/iconv.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #include <freetds/convert.h>
 #include <freetds/data.h>
+#include <freetds/utils.h>
 #include "replacements.h"
 
 #define TDS_ISSPACE(c) isspace((unsigned char) (c))
@@ -151,10 +152,8 @@ tsql_readline(char *prompt)
 		pos += strlen(line + pos);
 		if (pos + 1024 >= sz) {
 			sz += 1024;
-			p = (char*) realloc(line, sz);
-			if (!p)
+			if (!TDS_RESIZE(line, sz))
 				break;
-			line = p;
 		}
 	}
 	free(line);
@@ -311,9 +310,22 @@ static void
 tsql_print_usage(const char *progname)
 {
 	fprintf(stderr,
-		"Usage:\t%s [-a <appname>] [-S <server> | -H <hostname> -p <port>] -U <username> [-P <password>] [-I <config file>] [-o <options>] [-t delim] [-r delim] [-D database]\n"
-		"\t%s -C\n"
-		"Options:\n"
+		"Usage: %s [-a <appname>] [-S <server> | -H <hostname> -p <port>] -U <username> [-P <password>] [-I <config file>] [-o <options>] [-t delim] [-r delim] [-D database]\n"
+		"  or:  %s -C\n"
+		"  or:  %s -L -H <hostname>\n"
+		"If -C is specified just print configuration and exit.\n"
+		"If -L is specified with a host name (-H) instances found are printed.\n"
+		"  -a  specify application name\n"
+		"  -S  specify server entry in freetds.conf to connect\n"
+		"  -H  specify hostname to connect\n"
+		"  -p  specify port to connect to\n"
+		"  -U  specify username to use\n"
+		"  -P  specify password to use\n"
+		"  -D  specify database name to use\n"
+		"  -I  specify old configuration file (called interface) to use\n"
+		"  -J  specify character set to use\n"
+		"  -v  verbose mode\n"
+		"-o options:\n"
 		"\tf\tDo not print footer\n"
 		"\th\tDo not print header\n"
 		"\tt\tPrint time informations\n"
@@ -321,7 +333,7 @@ tsql_print_usage(const char *progname)
 		"\tq\tQuiet\n\n"
 		"\tDelimiters can be multi-char strings appropriately escaped for your shell.\n"
 		"\tDefault column delimitor is <tab>; default row delimiter is <newline>\n",
-		progname, progname);
+		progname, progname, progname);
 }
 
 static void
@@ -504,7 +516,7 @@ populate_login(TDSLOGIN * login, int argc, char **argv)
 			exit(0);
 			break;
 		default:
-			tsql_print_usage(argv[0]);
+			tsql_print_usage(basename(argv[0]));
 			exit(1);
 			break;
 		}
@@ -676,7 +688,10 @@ slurp_input_file(char *fname, char **mybuf, size_t *bufsz, size_t *buflen, int *
 	while ((s = fgets(linebuf, sizeof(linebuf), fp)) != NULL) {
 		while (*buflen + strlen(s) + 2 > *bufsz) {
 			*bufsz *= 2;
-			*mybuf = (char *) realloc(*mybuf, *bufsz);
+			if (!TDS_RESIZE(*mybuf, *bufsz)) {
+				perror("tsql: ");
+				exit(1);
+			}
 		}
 		strcpy(*mybuf + *buflen, s);
 		*buflen += strlen(*mybuf + *buflen);
@@ -699,6 +714,21 @@ print_instance_data(TDSLOGIN *login)
 		printf("connecting to instance %s on port %d\n", tds_dstr_cstr(&login->instance_name), login->port);
 }
 
+#if defined(HAVE_ALARM) && !defined(_WIN32)
+static void
+count_alarm(int s)
+{
+	static int count = 0;
+	char buf[64];
+
+	/* print the counter, do not use stderr as may be locked! */
+	sprintf(buf, "\r%2d", ++count);
+	write(STDERR_FILENO, buf, strlen(buf));
+
+	alarm(1);
+}
+#endif
+
 int
 main(int argc, char **argv)
 {
@@ -713,10 +743,6 @@ main(int argc, char **argv)
 	TDSCONTEXT *context;
 	TDSLOGIN *connection;
 	int opt_flags = 0;
-#if HAVE_FORK
-	pid_t timer_pid = 0;
-	int pipes[2];
-#endif
 	const char *locale = NULL;
 	const char *charset = NULL;
 
@@ -789,40 +815,11 @@ main(int argc, char **argv)
 	 * If we're able to establish an ip address for the server, we'll try to connect to it. 
 	 * If that machine is currently unreachable, show a timer connecting to the server. 
 	 */
-#if HAVE_FORK
+#if defined(HAVE_ALARM) && !defined(_WIN32)
 	if (connection && !QUIET) {
-		/*
-		 * We use a pipe so that the child is killed even main program aborts. 
-		 * Note that we do not read or write from this pipe. 
-		 */
-		if (pipe(pipes) < 0) {
-			perror("tsql: pipe");
-			return 1;
-		}
-
-		timer_pid = fork();
-		if (-1 == timer_pid) {
-			perror("tsql: warning"); /* make a note */
-		}
-		if (0 == timer_pid) {
-			int i=0;
-			close(pipes[1]);
-			while(1) {
-				fd_set set;
-				struct timeval tv;
-
-				FD_ZERO(&set);
-				FD_SET(pipes[0], &set);
-
-				tv.tv_sec = 1;
-				tv.tv_usec = 0;
-				if (select(pipes[0] + 1, &set, NULL, &set, &tv) != 0)
-					return 0;
-
-				fprintf(stderr, "\r%2d", ++i);
-			}
-		}
-		close(pipes[0]);
+		signal(SIGALRM, count_alarm);
+		fflush(stderr);
+		alarm(1);
 	}
 #endif
 	if (!connection || TDS_FAILED(tds_connect_and_login(tds, connection))) {
@@ -834,6 +831,15 @@ main(int argc, char **argv)
 		fprintf(stderr, "There was a problem connecting to the server\n");
 		exit(1);
 	}
+
+#if defined(HAVE_ALARM) && !defined(_WIN32)
+	if (!QUIET) {
+		alarm(0);
+		signal(SIGALRM, SIG_DFL);
+		fprintf(stderr, "\r");
+	}
+#endif
+
 	if (VERBOSE) 
 		print_instance_data(connection);
 	tds_free_login(connection);
@@ -847,22 +853,14 @@ main(int argc, char **argv)
 	rl_inhibit_completion = 1;
 #endif
 
-#if HAVE_FORK
-	if (timer_pid > 0 ) {
-		close(pipes[1]);
-		if (wait(0) == -1 ) {
-			perror("tsql: warning");
-		} else {
-			fprintf(stderr, "\r");
-		}
-	}
-#endif
-
 	for (s=NULL, s2=NULL; ; free(s), free(s2), s2=NULL) {
 		sprintf(prompt, "%d> ", ++line);
 		s = tsql_readline(QUIET ? NULL : prompt);
-		if (s == NULL) 
+		if (s == NULL) {
+			if (buflen)
+				do_query(tds, mybuf, global_opt_flags);
 			break;
+		}
 
 		/* 
 		 * 'GO' is special only at the start of a line
@@ -911,13 +909,11 @@ main(int argc, char **argv)
 			slurp_input_file(strtok(NULL, " \t"), &mybuf, &bufsz, &buflen, &line);
 		} else {
 			while (buflen + strlen(s) + 2 > bufsz) {
-				char *newbuf; 
 				bufsz *= 2;
-				if ((newbuf = (char *) realloc(mybuf, bufsz)) == NULL) {
+				if (!TDS_RESIZE(mybuf, bufsz)) {
 					perror("tsql: ");
 					exit(1);
 				}
-				mybuf = newbuf;
 			}
 			tsql_add_history(s);
 			strcpy(mybuf + buflen, s);
