@@ -331,6 +331,10 @@ cs_ctx_alloc(CS_INT version, CS_CONTEXT ** ctx)
 		/* set default in case there's no locale file */
 		tds_ctx->locale->date_fmt = strdup(STD_DATETIME_FMT);
 	}
+
+	(*ctx)->login_timeout = -1;
+	(*ctx)->query_timeout = -1;
+
 	return CS_SUCCEED;
 }
 
@@ -1289,6 +1293,8 @@ cs_will_convert(CS_CONTEXT * ctx, CS_INT srctype, CS_INT desttype, CS_BOOL * res
 
 	tdsdump_log(TDS_DBG_FUNC, "cs_will_convert(%p, %d, %d, %p)\n", ctx, srctype, desttype, result);
 
+	srctype = _ct_get_server_type(NULL, srctype);
+	desttype = _ct_get_server_type(NULL, desttype);
 	*result = (tds_willconvert(srctype, desttype) ? CS_TRUE : CS_FALSE);
 	return CS_SUCCEED;
 }
