@@ -1,6 +1,4 @@
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #if HAVE_STRING_H
 #include <string.h>
@@ -10,9 +8,6 @@
 #include <ctpublic.h>
 #include "common.h"
 
-static char software_version[] = "$Id: ct_diagclient.c,v 1.8 2007/04/12 07:06:12 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 /* Testing: Client Messages */
 int
 main(int argc, char *argv[])
@@ -28,7 +23,7 @@ main(int argc, char *argv[])
 	CS_INT num_cols;
 
 	CS_DATAFMT datafmt;
-	CS_INT datalength;
+	CS_INT datalength[2];
 	CS_SMALLINT ind[2];
 	CS_INT count, row_count = 0;
 	CS_INT cv;
@@ -124,7 +119,7 @@ main(int argc, char *argv[])
 
 			datafmt.count = 2;
 
-			ret = ct_bind(cmd, 1, &datafmt, &col1, &datalength, ind);
+			ret = ct_bind(cmd, 1, &datafmt, &col1[0], datalength, ind);
 			if (ret != CS_SUCCEED) {
 				fprintf(stderr, "ct_bind() failed\n");
 				return 1;
@@ -140,7 +135,7 @@ main(int argc, char *argv[])
 			datafmt.maxlength = 5;
 			datafmt.count = 4;
 
-			ret = ct_bind(cmd, 2, &datafmt, col2[0], &datalength, ind);
+			ret = ct_bind(cmd, 2, &datafmt, &col2[0], datalength, ind);
 			if (ret != CS_SUCCEED) {
 
 				if (ct_diag(conn, CS_STATUS, CS_CLIENTMSG_TYPE, CS_UNUSED, &num_msgs) != CS_SUCCEED) {
@@ -183,7 +178,7 @@ main(int argc, char *argv[])
 				return 1;
 			}
 			datafmt.count = 2;
-			ret = ct_bind(cmd, 2, &datafmt, col2[0], &datalength, ind);
+			ret = ct_bind(cmd, 2, &datafmt, &col2[0], datalength, ind);
 			if (ret != CS_SUCCEED) {
 				fprintf(stderr, "ct_bind() failed\n");
 				return 1;
