@@ -1,6 +1,4 @@
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdio.h>
 
@@ -18,7 +16,7 @@
 #define MAX(X,Y)      (((X) > (Y)) ? (X) : (Y))
 #define MIN(X,Y)      (((X) < (Y)) ? (X) : (Y))
 
-static char software_version[] = "$Id: rpc_ct_param.c,v 1.9 2009/07/21 06:44:19 freddy77 Exp $";
+static char software_version[] = "$Id: rpc_ct_param.c,v 1.10 2011-05-16 08:51:40 freddy77 Exp $";
 static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
 
 CS_RETCODE ex_clientmsg_cb(CS_CONTEXT * context, CS_CONNECTION * connection, CS_CLIENTMSG * errmsg);
@@ -57,6 +55,7 @@ main(int argc, char *argv[])
 	CS_FLOAT floatvar;
 	CS_MONEY moneyvar;
 	CS_BINARY binaryvar;
+	CS_BIT bitvar = 1;
 	char moneystring[10];
 	char rpc_name[15];
 	CS_INT destlen;
@@ -86,13 +85,13 @@ main(int argc, char *argv[])
 	ct_callback(ctx, NULL, CS_SET, CS_SERVERMSG_CB, (CS_VOID *) ex_servermsg_cb);
 
 	/* do not test error */
-	ret = run_command(cmd, "IF EXISTS(SELECT * FROM SYSOBJECTS WHERE name = 'sample_rpc' AND type = 'P') DROP PROCEDURE sample_rpc");
+	ret = run_command(cmd, "IF OBJECT_ID('sample_rpc') IS NOT NULL DROP PROCEDURE sample_rpc");
 
 	strcpy(cmdbuf, "create proc sample_rpc (@intparam int, \
         @sintparam smallint output, @floatparam float output, \
         @moneyparam money output,  \
         @dateparam datetime output, @charparam char(20) output, @empty varchar(20) output, \
-        @binaryparam    binary(20) output) \
+        @binaryparam    binary(20) output, @bitparam bit) \
         as ");
 
 	strcat(cmdbuf, "select @intparam, @sintparam, @floatparam, @moneyparam, \
@@ -282,6 +281,18 @@ main(int argc, char *argv[])
 		return 1;
 	}
 
+	strcpy(datafmt.name, "@bitparam");
+	datafmt.namelen = CS_NULLTERM;
+	datafmt.datatype = CS_BIT_TYPE;
+	datafmt.maxlength = 1;
+	datafmt.status = 0;
+	datafmt.locale = NULL;
+
+	if ((ret = ct_param(cmd, &datafmt, (CS_VOID *) & bitvar, CS_SIZEOF(CS_BIT), 0)) != CS_SUCCEED) {
+		fprintf(stderr, "ct_param(binary) failed");
+		return 1;
+	}
+
 	/*
 	 * Send the command to the server
 	 */
