@@ -46,7 +46,7 @@
 #include <freetds/tds.h>
 #include <freetds/thread.h>
 #include <freetds/convert.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #include <replacements.h>
 #include <sybfront.h>
 #include <sybdb.h>
@@ -107,6 +107,7 @@ col_init(struct col_t *pcol, int sybtype, int collen)
 	case SYBDATETIME4:
 	case SYBDATETIMN:
 		collen = 30;
+		/* fall through */
 	case SYBCHAR:
 	case SYBVARCHAR:
 	case SYBTEXT:
