@@ -1,4 +1,4 @@
-create table #dblib0016 (b1 bit, b2 bit, b3 bit, b4 bit)
+create table #dblib0016 (f1 int not null, f2 char(20) null, f3 binary(4) null)
 go
 select * from #dblib0016 where 0=1
 go
@@ -6,3 +6,4 @@ select * from #dblib0016 where 0=1
 go
 drop table #dblib0016
 go
+
