@@ -32,10 +32,11 @@
 
 #include <ctype.h>
 
+#include <freetds/utils.h>
 #include <freetds/odbc.h>
 #include <freetds/convert.h>
 #include <freetds/iconv.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #include <odbcss.h>
 
 #define TDS_ISSPACE(c) isspace((unsigned char) (c))
@@ -98,6 +99,7 @@ odbc_convert_char(TDS_STMT * stmt, TDSCOLUMN * curcol, TDS_CHAR * src, TDS_UINT
 		eat_iconv_left(curcol, &ob, &ol);
 		if (ol) {
 			memset(&conv->suppress, 0, sizeof(conv->suppress));
+			conv->suppress.eilseq = 1;
 			conv->suppress.e2big = 1;
 			/* TODO check return value */
 			tds_iconv(tds, conv, to_client, &ib, &il, &ob, &ol);
@@ -106,6 +108,9 @@ odbc_convert_char(TDS_STMT * stmt, TDSCOLUMN * curcol, TDS_CHAR * src, TDS_UINT
 		if (il && ol < sizeof(curcol->column_iconv_buf) && curcol->column_iconv_left == 0) {
 			char *left_ob = curcol->column_iconv_buf;
 			size_t left_ol = sizeof(curcol->column_iconv_buf);
+			conv->suppress.eilseq = 1;
+			conv->suppress.einval = 1;
+			conv->suppress.e2big = 1;
 			tds_iconv(tds, conv, to_client, &ib, &il, &left_ob, &left_ol);
 			curcol->column_iconv_left = sizeof(curcol->column_iconv_buf) - left_ol;
 			/* copy part to fill buffer */
@@ -269,7 +274,7 @@ odbc_convert_to_binary(TDS_STMT * stmt, TDSCOLUMN *curcol, int srctype, TDS_CHAR
 	return ret;
 }
 
-SQLLEN
+static SQLLEN
 odbc_tds2sql(TDS_STMT * stmt, TDSCOLUMN *curcol, int srctype, TDS_CHAR * src, TDS_UINT srclen, int desttype, TDS_CHAR * dest, SQLULEN destlen,
 	     const struct _drecord *drec_ixd)
 {
@@ -288,22 +293,6 @@ odbc_tds2sql(TDS_STMT * stmt, TDSCOLUMN *curcol, int srctype, TDS_CHAR * src, TD
 
 	assert(desttype != SQL_C_DEFAULT);
 
-	if (curcol) {
-		if (is_blob_col(curcol)) {
-			if (srctype == SYBLONGBINARY && (
-			    curcol->column_usertype == USER_UNICHAR_TYPE ||
-			    curcol->column_usertype == USER_UNIVARCHAR_TYPE))
-				srctype = SYBTEXT;
-			if (curcol->column_type == SYBVARIANT)
-				srctype = ((TDSVARIANT *) src)->type;
-			src = ((TDSBLOB *) src)->textvalue;
-		}
-		if (is_variable_type(curcol->column_type)) {
-			src += curcol->column_text_sqlgetdatapos;
-			srclen -= curcol->column_text_sqlgetdatapos;
-		}
-	}
-
 	nDestSybType = odbc_c_to_server_type(desttype);
 	if (!nDestSybType) {
 		odbc_errs_add(&stmt->errs, "HY003", NULL);
@@ -356,13 +345,7 @@ odbc_tds2sql(TDS_STMT * stmt, TDSCOLUMN *curcol, int srctype, TDS_CHAR * src, TD
 	if (desttype == SQL_C_WCHAR)
 		destlen /= sizeof(SQLWCHAR);
 	if (desttype == SQL_C_CHAR || desttype == SQL_C_WCHAR) {
-		switch (srctype) {
-		case SYBLONGBINARY:
-		case SYBBINARY:
-		case SYBVARBINARY:
-		case SYBIMAGE:
-		case XSYBBINARY:
-		case XSYBVARBINARY:
+		if (is_binary_type(srctype)) {
 			binary_conversion = 1;
 			if (destlen && !(destlen % 2))
 				--destlen;
@@ -629,3 +612,32 @@ normal_conversion:
 
 	return ret;
 }
+
+SQLLEN odbc_tds2sql_col(TDS_STMT * stmt, TDSCOLUMN *curcol, int desttype, TDS_CHAR * dest, SQLULEN destlen,
+			const struct _drecord *drec_ixd)
+{
+	int srctype = tds_get_conversion_type(curcol->on_server.column_type, curcol->on_server.column_size);
+	TDS_CHAR *src = (TDS_CHAR *) curcol->column_data;
+	TDS_UINT srclen = curcol->column_cur_size;
+
+	if (is_blob_col(curcol)) {
+		if (srctype == SYBLONGBINARY && (
+		    curcol->column_usertype == USER_UNICHAR_TYPE ||
+		    curcol->column_usertype == USER_UNIVARCHAR_TYPE))
+			srctype = SYBNTEXT;
+		if (curcol->column_type == SYBVARIANT)
+			srctype = ((TDSVARIANT *) src)->type;
+		src = ((TDSBLOB *) src)->textvalue;
+	}
+	if (is_variable_type(curcol->column_type)) {
+		src += curcol->column_text_sqlgetdatapos;
+		srclen -= curcol->column_text_sqlgetdatapos;
+	}
+	return odbc_tds2sql(stmt, curcol, srctype, src, srclen, desttype, dest, destlen, drec_ixd);
+}
+
+SQLLEN odbc_tds2sql_int4(TDS_STMT * stmt, TDS_INT *src, int desttype, TDS_CHAR * dest, SQLULEN destlen)
+{
+	return odbc_tds2sql(stmt, NULL, SYBINT4, (TDS_CHAR *) src, sizeof(*src),
+			    desttype, dest, destlen, NULL);
+}
