@@ -32,7 +32,7 @@
 #include <assert.h>
 
 #include <freetds/odbc.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 
 #if ENABLE_EXTRA_CHECKS
 
