@@ -32,7 +32,7 @@
 
 #include <freetds/odbc.h>
 #include <freetds/iconv.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #include <freetds/convert.h>
 #include <freetds/enum_cap.h>
 #include <odbcss.h>
@@ -499,8 +499,8 @@ odbc_set_return_status(struct _hstmt *stmt, unsigned int n_row)
 		}
 #define LEN(ptr) *((SQLLEN*)(((char*)(ptr)) + len_offset))
 
-		len = odbc_tds2sql(stmt, NULL, SYBINT4, (TDS_CHAR *) & tds->ret_status, sizeof(TDS_INT),
-				   drec->sql_desc_concise_type, (TDS_CHAR *) data_ptr, drec->sql_desc_octet_length, NULL);
+		len = odbc_tds2sql_int4(stmt, &tds->ret_status, drec->sql_desc_concise_type,
+					(TDS_CHAR *) data_ptr, drec->sql_desc_octet_length);
 		if (len == SQL_NULL_DATA)
 			return /* SQL_ERROR */ ;
 		if (drec->sql_desc_indicator_ptr)
@@ -530,8 +530,6 @@ odbc_set_return_params(struct _hstmt *stmt, unsigned int n_row)
 		const TDS_DESC* axd = stmt->apd;
 		const struct _drecord *drec_apd, *drec_ipd;
 		TDSCOLUMN *colinfo = info->columns[i];
-		TDS_CHAR *src;
-		int srclen;
 		SQLINTEGER len;
 		int c_type;
 		char *data_ptr;
@@ -569,10 +567,8 @@ odbc_set_return_params(struct _hstmt *stmt, unsigned int n_row)
 			continue;
 		}
 
-		src = (TDS_CHAR *) colinfo->column_data;
 		colinfo->column_text_sqlgetdatapos = 0;
 		colinfo->column_iconv_left = 0;
-		srclen = colinfo->column_cur_size;
 		c_type = drec_apd->sql_desc_concise_type;
 		if (c_type == SQL_C_DEFAULT)
 			c_type = odbc_sql_to_c_type_default(drec_ipd->sql_desc_concise_type);
@@ -580,8 +576,7 @@ odbc_set_return_params(struct _hstmt *stmt, unsigned int n_row)
 		 * TODO why IPD ?? perhaps SQLBindParameter it's not correct ??
 		 * Or tests are wrong ??
 		 */
-		len = odbc_tds2sql(stmt, colinfo, tds_get_conversion_type(colinfo->on_server.column_type, colinfo->on_server.column_size), src, srclen,
-				   c_type, (TDS_CHAR*) data_ptr, drec_apd->sql_desc_octet_length, drec_ipd);
+		len = odbc_tds2sql_col(stmt, colinfo, c_type, (TDS_CHAR*) data_ptr, drec_apd->sql_desc_octet_length, drec_ipd);
 		if (len == SQL_NULL_DATA)
 			return /* SQL_ERROR */ ;
 		if (drec_apd->sql_desc_indicator_ptr)
@@ -641,7 +636,7 @@ odbc_c_to_server_type(int c_type)
 	case SQL_C_TINYINT:
 	case SQL_C_UTINYINT:
 		return SYBINT1;
-		/* ODBC date formats are completely differect from SQL one */
+		/* ODBC date formats are completely different from SQL one */
 	case SQL_C_DATE:
 	case SQL_C_TIME:
 	case SQL_C_TIMESTAMP:
@@ -738,11 +733,13 @@ odbc_sql_to_server_type(TDSCONNECTION * conn, int sql_type, int sql_unsigned)
 	case SQL_WCHAR:
 		if (IS_TDS7_PLUS(conn))
 			return XSYBNCHAR;
+		/* fall thought */
 	case SQL_CHAR:
 		return SYBCHAR;
 	case SQL_WVARCHAR:
 		if (IS_TDS7_PLUS(conn))
 			return XSYBNVARCHAR;
+		/* fall thought */
 	case SQL_VARCHAR:
 		return SYBVARCHAR;
 	case SQL_SS_VARIANT:
@@ -754,6 +751,7 @@ odbc_sql_to_server_type(TDSCONNECTION * conn, int sql_type, int sql_unsigned)
 	case SQL_SS_XML:
 		if (IS_TDS72_PLUS(conn))
 			return SYBMSXML;
+		/* fall thought */
 	case SQL_WLONGVARCHAR:
 		if (IS_TDS7_PLUS(conn))
 			return SYBNTEXT;
@@ -771,8 +769,7 @@ odbc_sql_to_server_type(TDSCONNECTION * conn, int sql_type, int sql_unsigned)
 		return TDS_INVALID_TYPE;
 #endif
 	case SQL_BIT:
-		if (IS_TDS7_PLUS(conn))
-			return SYBBITN;
+		/* NOTE: always return not nullable type */
 		return SYBBIT;
 	case SQL_TINYINT:
 		return SYBINT1;
@@ -813,6 +810,7 @@ odbc_sql_to_server_type(TDSCONNECTION * conn, int sql_type, int sql_unsigned)
 			return SYBTIME;
 		if (IS_TDS73_PLUS(conn))
 			return SYBMSTIME;
+		/* fall thought */
 	type_timestamp:
 	case SQL_TYPE_TIMESTAMP:
 		if (IS_TDS73_PLUS(conn))
