@@ -18,9 +18,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -36,22 +34,17 @@
 
 #include <ctype.h>
 
-#include "tdsodbc.h"
-#include "tdsconvert.h"
-
-#ifdef DMALLOC
-#include <dmalloc.h>
-#endif
-
-TDS_RCSID(var, "$Id: prepare_query.c,v 1.79 2009/12/02 22:58:21 jklowden Exp $");
+#include <freetds/odbc.h>
+#include <freetds/convert.h>
 
 #define TDS_ISSPACE(c) isspace((unsigned char) (c))
 
 static int
-prepared_rpc(struct _hstmt *stmt, int compute_row)
+prepared_rpc(struct _hstmt *stmt, bool compute_row)
 {
 	int nparam = stmt->params ? stmt->params->num_cols : 0;
 	const char *p = stmt->prepared_pos - 1;
+	TDSCONNECTION *conn = stmt->dbc->tds_socket->conn;
 
 	for (;;) {
 		TDSPARAMINFO *temp_params;
@@ -59,7 +52,8 @@ prepared_rpc(struct _hstmt *stmt, int compute_row)
 		TDS_SERVER_TYPE type;
 		const char *start;
 
-		while (TDS_ISSPACE(*++p));
+		while (TDS_ISSPACE(*++p))
+			continue;
 		if (!*p)
 			return SQL_SUCCESS;
 
@@ -73,12 +67,12 @@ prepared_rpc(struct _hstmt *stmt, int compute_row)
 
 		switch (*p) {
 		case ',':
-			if (IS_TDS7_PLUS(stmt->dbc->tds_socket)) {
-				tds_set_param_type(stmt->dbc->tds_socket, curcol, SYBVOID);
+			if (IS_TDS7_PLUS(conn)) {
+				tds_set_param_type(conn, curcol, SYBVOID);
 				curcol->column_size = curcol->column_cur_size = 0;
 			} else {
 				/* TODO is there a better type ? */
-				tds_set_param_type(stmt->dbc->tds_socket, curcol, SYBINTN);
+				tds_set_param_type(conn, curcol, SYBINTN);
 				curcol->column_size = curcol->on_server.column_size = 4;
 				curcol->column_cur_size = -1;
 			}
@@ -97,7 +91,7 @@ prepared_rpc(struct _hstmt *stmt, int compute_row)
 				tds_free_param_result(temp_params);
 				return SQL_ERROR;
 			}
-			tds_set_param_type(stmt->dbc->tds_socket, curcol, type);
+			tds_set_param_type(conn, curcol, type);
 			switch (type) {
 			case SYBVARCHAR:
 				curcol->column_size = p - start;
@@ -108,6 +102,7 @@ prepared_rpc(struct _hstmt *stmt, int compute_row)
 			default:
 				assert(0);
 			case SYBINT4:
+			case SYBINT8:
 			case SYBFLT8:
 				curcol->column_cur_size = curcol->column_size;
 				break;
@@ -152,6 +147,9 @@ prepared_rpc(struct _hstmt *stmt, int compute_row)
 				case SYBINT4:
 					*((TDS_INT *) dest) = strtol(start, NULL, 10);
 					break;
+				case SYBINT8:
+					*((TDS_INT8 *) dest) = tds_strtoll(start, NULL, 10);
+					break;
 				case SYBFLT8:
 					*((TDS_FLOAT *) dest) = strtod(start, NULL);
 					break;
@@ -162,7 +160,7 @@ prepared_rpc(struct _hstmt *stmt, int compute_row)
 			--p;
 			break;
 		case '?':
-			/* find binded parameter */
+			/* find bound parameter */
 			if (stmt->param_num > stmt->apd->header.sql_desc_count
 			    || stmt->param_num > stmt->ipd->header.sql_desc_count) {
 				tds_free_param_result(temp_params);
@@ -183,7 +181,8 @@ prepared_rpc(struct _hstmt *stmt, int compute_row)
 		}
 		++nparam;
 
-		while (TDS_ISSPACE(*++p));
+		while (TDS_ISSPACE(*++p))
+			continue;
 		if (!*p || *p != ',')
 			return SQL_SUCCESS;
 		stmt->prepared_pos = (char *) p + 1;
@@ -191,7 +190,7 @@ prepared_rpc(struct _hstmt *stmt, int compute_row)
 }
 
 int
-parse_prepared_query(struct _hstmt *stmt, int compute_row)
+parse_prepared_query(struct _hstmt *stmt, bool compute_row)
 {
 	/* try setting this parameter */
 	TDSPARAMINFO *temp_params;
@@ -232,7 +231,7 @@ parse_prepared_query(struct _hstmt *stmt, int compute_row)
 }
 
 int
-start_parse_prepared_query(struct _hstmt *stmt, int compute_row)
+start_parse_prepared_query(struct _hstmt *stmt, bool compute_row)
 {
 	/* TODO should be NULL already ?? */
 	tds_free_param_results(stmt->params);
@@ -370,13 +369,8 @@ continue_parse_prepared_query(struct _hstmt *stmt, SQLPOINTER DataPtr, SQLLEN St
 		SQLLEN orig_len = len;
 
 		if (sql_src_type == SQL_C_CHAR || sql_src_type == SQL_C_WCHAR) {
-			switch (tds_get_conversion_type(curcol->column_type, curcol->column_size)) {
-			case SYBBINARY:
-			case SYBVARBINARY:
-			case XSYBBINARY:
-			case XSYBVARBINARY:
-			case SYBLONGBINARY:
-			case SYBIMAGE:
+			TDS_SERVER_TYPE type = tds_get_conversion_type(curcol->column_type, curcol->column_size);
+			if (is_binary_type(type)) {
 				if (len && sql_src_type == SQL_C_CHAR && !*((char*)DataPtr+len-1))
 					--len;
 
@@ -389,21 +383,18 @@ continue_parse_prepared_query(struct _hstmt *stmt, SQLPOINTER DataPtr, SQLLEN St
 				binary_convert = 1;
 				orig_len = len;
 				len = len / 2u + 1u;
-				break;
 			}
 		}
 
-		if (blob->textvalue)
-			p = (TDS_CHAR *) realloc(blob->textvalue, len + curcol->column_cur_size);
-		else {
-			assert(curcol->column_cur_size == 0);
-			p = (TDS_CHAR *) malloc(len);
-		}
+		if (!len)
+			return SQL_SUCCESS;
+
+		assert(blob->textvalue || curcol->column_cur_size == 0);
+		p = (TDS_CHAR *) TDS_RESIZE(blob->textvalue, len + curcol->column_cur_size);
 		if (!p) {
 			odbc_errs_add(&stmt->errs, "HY001", NULL); /* Memory allocation error */
 			return SQL_ERROR;
 		}
-		blob->textvalue = p;
 
 		p += curcol->column_cur_size;
 		if (binary_convert) {
@@ -435,8 +426,8 @@ continue_parse_prepared_query(struct _hstmt *stmt, SQLPOINTER DataPtr, SQLLEN St
 			}
 
 			res = (sql_src_type == SQL_C_CHAR) ?
-				tds_char2hex(p, len / 2u, DataPtr, len):
-				odbc_wchar2hex(p, len / 2u, DataPtr, len);
+				tds_char2hex(p, len / 2u, (const TDS_CHAR*) DataPtr, len):
+				odbc_wchar2hex(p, len / 2u, (const SQLWCHAR*) DataPtr, len);
 			if (res < 0) {
 				odbc_convert_err_set(&stmt->errs, res);
 				return SQL_ERROR;
