@@ -3,23 +3,11 @@
 
 /* Test using array binding */
 
-static char software_version[] = "$Id: array.c,v 1.16 2010/07/05 09:20:32 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+static SQLTCHAR *test_query = NULL;
+static int multiply = 90;
+static int failure = 0;
 
-static const char *test_query = NULL;
-
-static void *
-xmalloc(size_t s)
-{
-	void *p = malloc(s);
-	if (!p) {
-		fprintf(stderr, "Out of memory\n");
-		exit(1);
-	}
-	return p;
-}
-
-#define XMALLOC_N(t, n) (t*) xmalloc(n*sizeof(t))
+#define XMALLOC_N(t, n) (t*) ODBC_GET(n*sizeof(t))
 
 static void
 query_test(int prepare, SQLRETURN expected, const char *expected_status)
@@ -32,10 +20,12 @@ query_test(int prepare, SQLRETURN expected, const char *expected_status)
 	desc_t *descs = XMALLOC_N(desc_t, ARRAY_SIZE);
 	SQLLEN *id_lens = XMALLOC_N(SQLLEN,ARRAY_SIZE), *desc_lens = XMALLOC_N(SQLLEN,ARRAY_SIZE);
 	SQLUSMALLINT i, *statuses = XMALLOC_N(SQLUSMALLINT,ARRAY_SIZE);
+	unsigned *num_errors = XMALLOC_N(unsigned, ARRAY_SIZE);
 	SQLULEN processed;
 	RETCODE ret;
 	char status[20];
-	int failure = 0;
+	SQLTCHAR *err = (SQLTCHAR *) ODBC_GET(sizeof(odbc_err)*sizeof(SQLTCHAR));
+	SQLTCHAR *state = (SQLTCHAR *) ODBC_GET(sizeof(odbc_sqlstate)*sizeof(SQLTCHAR));
 
 	assert(odbc_stmt != SQL_NULL_HSTMT);
 	odbc_reset_statement();
@@ -53,27 +43,20 @@ query_test(int prepare, SQLRETURN expected, const char *expected_status)
 	processed = ARRAY_SIZE + 1;
 	for (i = 0; i < ARRAY_SIZE; i++) {
 		statuses[i] = SQL_PARAM_DIAG_UNAVAILABLE;
-		ids[i] = i * 132;
+		ids[i] = (i + 1) * multiply;
 		sprintf((char *) descs[i], "data %d", i * 7);
 		id_lens[i] = 0;
 		desc_lens[i] = SQL_NTS;
+		num_errors[i] = 0;
 	}
+	multiply = 90;
 
 	if (!prepare) {
-		ret = SQLExecDirect(odbc_stmt, (SQLCHAR *) test_query, SQL_NTS);
+		ret = SQLExecDirect(odbc_stmt, test_query, SQL_NTS);
 	} else {
-		SQLPrepare(odbc_stmt, (SQLCHAR *) test_query, SQL_NTS);
+		SQLPrepare(odbc_stmt, test_query, SQL_NTS);
 		ret = SQLExecute(odbc_stmt);
 	}
-	if (ret != expected) {
-		char buf[256];
-
-		sprintf(buf, "Invalid result: got %d exptected %d processed %d", ret, expected, (int) processed);
-		ODBC_REPORT_ERROR(buf);
-	}
-
-	for (i = 0; i < ARRAY_SIZE; i++)
-		SQLMoreResults(odbc_stmt);
 
 	if (processed > ARRAY_SIZE) {
 		char buf[256];
@@ -82,14 +65,34 @@ query_test(int prepare, SQLRETURN expected, const char *expected_status)
 		ODBC_REPORT_ERROR(buf);
 	}
 
+	for (i = 1; CHKGetDiagRec(SQL_HANDLE_STMT, odbc_stmt, i, state, NULL, err, sizeof(odbc_err), NULL, "SINo") != SQL_NO_DATA; ++i) {
+		SQLINTEGER row;
+
+		strcpy(odbc_err, C(err));
+		strcpy(odbc_sqlstate, C(state));
+		CHKGetDiagField(SQL_HANDLE_STMT, odbc_stmt, i, SQL_DIAG_ROW_NUMBER, &row, sizeof(row), NULL, "S");
+
+		if (row == SQL_ROW_NUMBER_UNKNOWN) continue;
+		if (row < 1 || row > ARRAY_SIZE) {
+			fprintf(stderr, "invalid row %d returned reading error number %d\n", (int) row, i);
+			exit(1);
+		}
+		++num_errors[row-1];
+		printf("for row %2d returned '%s' %s\n", (int) row, odbc_sqlstate, odbc_err);
+	}
+
 	for (i = 0; i < processed; ++i) {
+		int has_diag = 0;
+
 		switch (statuses[i]) {
-		case SQL_PARAM_SUCCESS:
 		case SQL_PARAM_SUCCESS_WITH_INFO:
+			has_diag = 1;
+		case SQL_PARAM_SUCCESS:
 			status[i] = 'V';
 			break;
 
 		case SQL_PARAM_ERROR:
+			has_diag = 1;
 			status[i] = '!';
 			break;
 
@@ -101,28 +104,35 @@ query_test(int prepare, SQLRETURN expected, const char *expected_status)
 			status[i] = '?';
 			break;
 		default:
-			fprintf(stderr, "Invalid status returned\n");
+			fprintf(stderr, "Invalid status returned %d\n", statuses[i]);
 			exit(1);
 		}
+
+		if (has_diag) {
+			if (!num_errors[i]) {
+				fprintf(stderr, "Diagnostics not returned for status %d\n", i);
+				failure = 1;
+			}
+		} else {
+			if (num_errors[i]) {
+				fprintf(stderr, "Diagnostics returned for status %d\n", i);
+				failure = 1;
+			}
+		}
 	}
 	status[i] = 0;
 
-	if (expected_status && strcmp(expected_status, status) != 0) {
-		fprintf(stderr, "Invalid status\n\tgot      '%s'\n\texpected '%s'\n", status, expected_status);
+	if (ret != expected || strcmp(expected_status, status) != 0) {
+		fprintf(stderr, "Invalid result: got %d \"%s\" expected %d \"%s\" processed %d\n",
+			ret, status, expected, expected_status, (int) processed);
+		if (ret != SQL_SUCCESS)
+			odbc_read_error();
 		failure = 1;
+		odbc_reset_statement();
+		return;
 	}
 
 	odbc_reset_statement();
-
-	free(ids);
-	free(descs);
-	free(id_lens);
-	free(desc_lens);
-	free(statuses);
-	if (failure) {
-		odbc_disconnect();
-		exit(1);
-	}
 }
 
 int
@@ -132,26 +142,42 @@ main(int argc, char *argv[])
 	odbc_connect();
 
 	if (odbc_db_is_microsoft()) {
-		test_query = "INSERT INTO #tmp1 (id, value) VALUES (?, ?)";
+		/* all successes */
+		test_query = T("INSERT INTO #tmp1 (id, value) VALUES (?, ?)");
+		multiply = 1;
+		query_test(0, SQL_SUCCESS, "VVVVVVVVVV");
+		multiply = 1;
+		query_test(1, SQL_SUCCESS, "VVVVVVVVVV");
+
+		/* all errors */
+		test_query = T("INSERT INTO #tmp1 (id, value) VALUES (?, ?)");
+		multiply = 257;
+		query_test(0, SQL_ERROR, "!!!!!!!!!!");
+		multiply = 257;
+		query_test(1, SQL_SUCCESS_WITH_INFO, "!!!!!!!!!!");
+
+		test_query = T("INSERT INTO #tmp1 (id, value) VALUES (?, ?)");
 		query_test(0, SQL_ERROR, "VV!!!!!!!!");
-		/* FIXME test why is different and what should be correct result */
-		query_test(1, odbc_driver_is_freetds() ? SQL_ERROR : SQL_SUCCESS_WITH_INFO, "VV!!!!!!!!");
+		query_test(1, SQL_SUCCESS_WITH_INFO, "VV!!!!!!!!");
 
-		test_query = "INSERT INTO #tmp1 (id) VALUES (?) UPDATE #tmp1 SET value = ?";
+		test_query = T("INSERT INTO #tmp1 (id, value) VALUES (900-?, ?)");
+		query_test(0, SQL_SUCCESS_WITH_INFO, "!!!!!!!VVV");
+		query_test(1, SQL_SUCCESS_WITH_INFO, "!!!!!!!VVV");
+
+		test_query = T("INSERT INTO #tmp1 (id) VALUES (?) UPDATE #tmp1 SET value = ?");
 		query_test(0, SQL_SUCCESS_WITH_INFO, "VVVV!V!V!V");
-		/* FIXME test why is different and what should be correct result */
-		query_test(1, odbc_driver_is_freetds() ? SQL_ERROR : SQL_SUCCESS_WITH_INFO, "VV!!!!!!!!");
+		query_test(1, SQL_SUCCESS_WITH_INFO, "VV!!!!!!!!");
 
+#ifdef ENABLE_DEVELOPING
 		/* with result, see how SQLMoreResult work */
-		test_query = "INSERT INTO #tmp1 (id) VALUES (?) SELECT * FROM #tmp1 UPDATE #tmp1 SET value = ?";
+		test_query = T("INSERT INTO #tmp1 (id) VALUES (?) SELECT * FROM #tmp1 UPDATE #tmp1 SET value = ?");
 		/* IMHO our driver is better here -- freddy77 */
 		query_test(0, SQL_SUCCESS, odbc_driver_is_freetds() ? "VVVVV!V!V!" : "VVVVVV!VVV");
-#ifdef ENABLE_DEVELOPING
 		query_test(1, SQL_SUCCESS, "VVVVVVVVVV");
 #endif
 	} else {
 		/* Sybase test for conversions before executing */
-		test_query = "INSERT INTO #tmp1 (id, value) VALUES (?/8, ?)";
+		test_query = T("INSERT INTO #tmp1 (id, value) VALUES (?/8, ?)");
 		query_test(0, SQL_SUCCESS, "VVVVVVVVVV");
 	}
 
@@ -159,7 +185,7 @@ main(int argc, char *argv[])
 
 	odbc_disconnect();
 
-	printf("Success!.\n");
-	return 0;
+	printf(failure ? "Failed :(\n" : "Success!\n");
+	return failure;
 }
 
