@@ -9,6 +9,7 @@
 #endif /* HAVE_UNISTD_H */
 
 #include <freetds/thread.h>
+#include <freetds/utils.h>
 #include "replacements.h"
 
 #if TDS_HAVE_MUTEX
@@ -80,13 +81,13 @@ static TDS_THREAD_PROC_DECLARE(wait_thread_proc, arg)
 		tds_mutex_lock(&mtx);
 		if (exit_thread) {
 			tds_mutex_unlock(&mtx);
-			return NULL;
+			return TDS_THREAD_RESULT(0);
 		}
 		tds_mutex_unlock(&mtx);
 	}
 
 	exit_forced(0);
-	return NULL;
+	return TDS_THREAD_RESULT(0);
 }
 
 static void
