@@ -37,6 +37,10 @@ char odbc_password[512];
 char odbc_database[512];
 char odbc_driver[1024];
 
+static int freetds_driver = -1;
+static int tds_version = -1;
+static char db_str_version[32];
+
 static int
 check_lib(char *path, const char *file)
 {
@@ -91,7 +95,8 @@ odbc_read_login_info(void)
 	const char *const *search_p;
 	char path[1024];
 	int len;
-#ifdef _WIN32
+	int ini_override = 1;
+#if defined(_WIN32) && !defined(TDS_NO_DM)
 	UWORD old_config_mode;
 #endif
 
@@ -157,25 +162,33 @@ odbc_read_login_info(void)
 		return 0;
 	strcpy(odbc_driver, path);
 
-#ifndef _WIN32
+	s1 = getenv("TDSINIOVERRIDE");
+	if (s1 && atoi(s1) == 0)
+		ini_override = 0;
+
+#if !defined(_WIN32) || defined(TDS_NO_DM)
 	/* craft out odbc.ini, avoid to read wrong one */
 	sprintf(path, "odbc.ini.%d", (int) getpid());
 	in = fopen(path, "w");
 	if (in) {
 		fprintf(in, "[%s]\nDriver = %s\nDatabase = %s\nServername = %s\n", odbc_server, odbc_driver, odbc_database, odbc_server);
 		fclose(in);
-		setenv("ODBCINI", "./odbc.ini", 1);
-		setenv("SYSODBCINI", "./odbc.ini", 1);
-		rename(path, "odbc.ini");
+		if (ini_override) {
+			setenv("ODBCINI", "./odbc.ini", 1);
+			setenv("SYSODBCINI", "./odbc.ini", 1);
+			rename(path, "odbc.ini");
+		}
 		unlink(path);
 	}
 #else
-	if (SQLGetConfigMode(&old_config_mode)) {
+	if (ini_override && SQLGetConfigMode(&old_config_mode)) {
+		ODBC_BUF *odbc_buf = NULL;
 		SQLSetConfigMode(ODBC_USER_DSN);
-		SQLWritePrivateProfileString(odbc_server, "Driver", odbc_driver, "odbc.ini");
-		SQLWritePrivateProfileString(odbc_server, "Database", odbc_database, "odbc.ini");
-		SQLWritePrivateProfileString(odbc_server, "Servername", odbc_server, "odbc.ini");
+		SQLWritePrivateProfileString(T(odbc_server), T("Driver"), T(odbc_driver), T("odbc.ini"));
+		SQLWritePrivateProfileString(T(odbc_server), T("Database"), T(odbc_database), T("odbc.ini"));
+		SQLWritePrivateProfileString(T(odbc_server), T("Servername"), T(odbc_server), T("odbc.ini"));
 		SQLSetConfigMode(old_config_mode);
+		ODBC_FREE();
 	}
 #endif
 	return 0;
@@ -241,7 +254,7 @@ int
 odbc_connect(void)
 {
 	ODBC_BUF *odbc_buf = NULL;
-	char command[512];
+	char command[512+10];
 	const char *p;
 
 	if (odbc_read_login_info())
@@ -302,6 +315,10 @@ odbc_disconnect(void)
 		odbc_env = SQL_NULL_HENV;
 	}
 	ODBC_FREE();
+
+	freetds_driver = -1;
+	tds_version = -1;
+	db_str_version[0] = 0;
 	return 0;
 }
 
@@ -337,7 +354,6 @@ odbc_db_is_microsoft(void)
 	return ms_db;
 }
 
-static int freetds_driver = -1;
 int
 odbc_driver_is_freetds(void)
 {
@@ -440,8 +456,6 @@ odbc_tds_version_long(void)
 int
 odbc_tds_version(void)
 {
-	static int tds_version = -1;
-
 	ODBC_BUF *odbc_buf = NULL;
 	SQLUINTEGER version;
 	SQLSMALLINT len;
@@ -460,8 +474,6 @@ odbc_tds_version(void)
 	return tds_version;
 }
 
-static char db_str_version[32];
-
 const char *odbc_db_version(void)
 {
 	if (!db_str_version[0]) {
@@ -807,6 +819,13 @@ odbc_find_last_socket(void)
 	unsigned num_found = 0, n;
 	int i;
 
+	SQLULEN sock = 0;
+	SQLSMALLINT len;
+	if (odbc_driver_is_freetds()
+	    && SQLGetInfo(odbc_conn, 1301 /* SQL_INFO_FREETDS_SOCKET */, &sock, sizeof(sock), &len) == SQL_SUCCESS
+	    && len == sizeof(sock))
+		return (TDS_SYS_SOCKET) sock;
+
 	FOR_ALL_SOCKETS(i) {
 		long_sockaddr remote_addr, local_addr;
 		struct sockaddr_in *in;
@@ -892,3 +911,68 @@ odbc_check_no_row(const char *query)
 		}
 	} while (CHKMoreResults("SNo") == SQL_SUCCESS);
 }
+
+int
+odbc_lookup(const char *name, const struct odbc_lookup_int *table, int def)
+{
+	for (; table->name; ++table)
+		if (strcmp(table->name, name) == 0)
+			return table->value;
+
+	return def;
+}
+
+const char*
+odbc_lookup_value(int value, const struct odbc_lookup_int *table, const char *def)
+{
+	for (; table->name; ++table)
+		if (table->value == value)
+			return table->name;
+
+	return def;
+}
+
+struct odbc_lookup_int odbc_sql_c_types[] = {
+#define TYPE(s) { #s, s }
+	TYPE(SQL_C_NUMERIC),
+	TYPE(SQL_C_BINARY),
+	TYPE(SQL_C_CHAR),
+	TYPE(SQL_C_WCHAR),
+	TYPE(SQL_C_LONG),
+	TYPE(SQL_C_SBIGINT),
+	TYPE(SQL_C_SHORT),
+	TYPE(SQL_C_TIMESTAMP),
+	TYPE(SQL_C_FLOAT),
+	TYPE(SQL_C_DOUBLE),
+	TYPE(SQL_C_DEFAULT),
+	TYPE(SQL_C_DATE),
+	TYPE(SQL_C_TIME),
+	TYPE(SQL_C_TYPE_DATE),
+	TYPE(SQL_C_TYPE_TIME),
+	TYPE(SQL_C_TYPE_TIMESTAMP),
+	TYPE(SQL_C_INTERVAL_YEAR),
+	TYPE(SQL_C_INTERVAL_MONTH),
+	TYPE(SQL_C_INTERVAL_DAY),
+	TYPE(SQL_C_INTERVAL_HOUR),
+	TYPE(SQL_C_INTERVAL_MINUTE),
+	TYPE(SQL_C_INTERVAL_SECOND),
+	TYPE(SQL_C_INTERVAL_YEAR_TO_MONTH),
+	TYPE(SQL_C_INTERVAL_DAY_TO_HOUR),
+	TYPE(SQL_C_INTERVAL_DAY_TO_MINUTE),
+	TYPE(SQL_C_INTERVAL_DAY_TO_SECOND),
+	TYPE(SQL_C_INTERVAL_HOUR_TO_MINUTE),
+	TYPE(SQL_C_INTERVAL_HOUR_TO_SECOND),
+	TYPE(SQL_C_INTERVAL_MINUTE_TO_SECOND),
+	TYPE(SQL_C_BIT),
+	TYPE(SQL_C_UBIGINT),
+	TYPE(SQL_C_TINYINT),
+	TYPE(SQL_C_SLONG),
+	TYPE(SQL_C_SSHORT),
+	TYPE(SQL_C_STINYINT),
+	TYPE(SQL_C_ULONG),
+	TYPE(SQL_C_USHORT),
+	TYPE(SQL_C_UTINYINT),
+	TYPE(SQL_C_GUID),
+#undef TYPE
+	{ NULL, 0 }
+};
