@@ -1,16 +1,18 @@
 #ifdef _WIN32
-#include <winsock2.h>
-#include <windows.h>
+#define _CRT_SECURE_NO_WARNINGS 1
+#include <freetds/windows.h>
 #include <direct.h>
 #endif
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
 
+#if HAVE_UNISTD_H
+#include <unistd.h>
+#endif /* HAVE_UNISTD_H */
+
 #if HAVE_STDLIB_H
 #include <stdlib.h>
 #endif /* HAVE_STDLIB_H */
@@ -21,9 +23,9 @@
 
 #include <sql.h>
 #include <sqlext.h>
+#include <sqlucode.h>
 
-static char rcsid_common_h[] = "$Id: common.h,v 1.35 2010/09/01 08:39:38 freddy77 Exp $";
-static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_warn };
+#include <freetds/sysdep_private.h>
 
 #ifndef HAVE_SQLLEN
 #ifndef SQLULEN
@@ -34,6 +36,8 @@ static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_wa
 #endif
 #endif
 
+#define FREETDS_SRCDIR FREETDS_TOPDIR "/src/odbc/unittests"
+
 extern HENV odbc_env;
 extern HDBC odbc_conn;
 extern HSTMT odbc_stmt;
@@ -61,6 +65,8 @@ void odbc_check_rows(int n, int line, const char * file);
 void odbc_reset_statement_proc(SQLHSTMT *stmt, const char *file, int line);
 #define odbc_reset_statement() odbc_reset_statement_proc(&odbc_stmt, __FILE__, __LINE__)
 void odbc_check_cursor(void);
+void odbc_check_no_row(const char *query);
+void odbc_test_skipped(void);
 
 #define ODBC_REPORT_ERROR(msg) odbc_report_error(msg, __LINE__, __FILE__)
 
@@ -92,6 +98,8 @@ SQLSMALLINT odbc_alloc_handle_err_type(SQLSMALLINT type);
 	CHKR2(SQLColAttribute, (odbc_stmt,a,b,c,d,e,f), SQL_HANDLE_STMT, odbc_stmt, res)
 #define CHKDescribeCol(a,b,c,d,e,f,g,h,res) \
 	CHKR2(SQLDescribeCol, (odbc_stmt,a,b,c,d,e,f,g,h), SQL_HANDLE_STMT, odbc_stmt, res)
+#define CHKConnect(a,b,c,d,e,f,res) \
+	CHKR2(SQLConnect, (odbc_conn,a,b,c,d,e,f), SQL_HANDLE_DBC, odbc_conn, res)
 #define CHKDriverConnect(a,b,c,d,e,f,g,res) \
 	CHKR2(SQLDriverConnect, (odbc_conn,a,b,c,d,e,f,g), SQL_HANDLE_DBC, odbc_conn, res)
 #define CHKEndTran(a,b,c,res) \
@@ -114,6 +122,8 @@ SQLSMALLINT odbc_alloc_handle_err_type(SQLSMALLINT type);
 	CHKR2(SQLGetConnectAttr, (odbc_conn,a,b,c,d), SQL_HANDLE_DBC, odbc_conn, res)
 #define CHKGetDiagRec(a,b,c,d,e,f,g,h,res) \
 	CHKR2(SQLGetDiagRec, (a,b,c,d,e,f,g,h), a, b, res)
+#define CHKGetDiagField(a,b,c,d,e,f,g,res) \
+	CHKR2(SQLGetDiagField, (a,b,c,d,e,f,g), a, b, res)
 #define CHKGetStmtAttr(a,b,c,d,res) \
 	CHKR2(SQLGetStmtAttr, (odbc_stmt,a,b,c,d), SQL_HANDLE_STMT, odbc_stmt, res)
 #define CHKGetTypeInfo(a,res) \
@@ -163,7 +173,9 @@ int odbc_db_is_microsoft(void);
 const char *odbc_db_version(void);
 unsigned int odbc_db_version_int(void);
 int odbc_driver_is_freetds(void);
+int odbc_tds_version(void);
 
+#define ODBC_VECTOR_SIZE(x) (sizeof(x)/sizeof(x[0]))
 #define int2ptr(i) ((void*)(((char*)0)+(i)))
 #define ptr2int(p) ((int)(((char*)(p))-((char*)0)))
 
@@ -173,6 +185,48 @@ void odbc_setenv(const char *name, const char *value, int overwrite);
 #define setenv odbc_setenv
 #endif
 
+void odbc_mark_sockets_opened(void);
+TDS_SYS_SOCKET odbc_find_last_socket(void);
+
+/**
+ * Converts an ODBC result into a string.
+ * There is no check on destination length, use a buffer big enough.
+ */
+void odbc_c2string(char *out, SQLSMALLINT out_c_type, const void *in, size_t in_len);
+
 int odbc_to_sqlwchar(SQLWCHAR *dst, const char *src, int n);
 int odbc_from_sqlwchar(char *dst, const SQLWCHAR *src, int n);
 
+typedef struct odbc_buf{
+	struct odbc_buf *next;
+	void *buf;
+} ODBC_BUF;
+extern ODBC_BUF *odbc_buf;
+void *odbc_buf_get(ODBC_BUF** buf, size_t s);
+void odbc_buf_free(ODBC_BUF** buf);
+#define ODBC_GET(s)  odbc_buf_get(&odbc_buf, s)
+#define ODBC_FREE() odbc_buf_free(&odbc_buf)
+
+SQLWCHAR *odbc_get_sqlwchar(ODBC_BUF** buf, const char *s);
+char *odbc_get_sqlchar(ODBC_BUF** buf, SQLWCHAR *s);
+
+#undef T
+#ifdef UNICODE
+/* char to TCHAR */
+#define T(s) odbc_get_sqlwchar(&odbc_buf, (s))
+/* TCHAR to char */
+#define C(s) odbc_get_sqlchar(&odbc_buf, (s))
+#else
+#define T(s) ((SQLCHAR*)(s))
+#define C(s) ((char*)(s))
+#endif
+
+struct odbc_lookup_int
+{
+	const char *name;
+	int value;
+};
+
+int odbc_lookup(const char *name, const struct odbc_lookup_int *table, int def);
+const char *odbc_lookup_value(int value, const struct odbc_lookup_int *table, const char *def);
+extern struct odbc_lookup_int odbc_sql_c_types[];
