@@ -1,23 +1,10 @@
 #include "common.h"
 
-/* TODO port to windows, use thread */
-
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#if TIME_WITH_SYS_TIME
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# endif
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
+#include <freetds/time.h>
 
 #if HAVE_ERRNO_H
 #include <errno.h>
@@ -43,72 +30,33 @@
 #include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
 
-#if (defined(TDS_HAVE_PTHREAD_MUTEX) && HAVE_ALARM && HAVE_FSTAT && defined(S_IFSOCK)) || defined(_WIN32)
+#if (!defined(TDS_NO_THREADSAFE) && HAVE_ALARM && HAVE_FSTAT && defined(S_IFSOCK)) || defined(_WIN32)
 
 #include <ctype.h>
-#if HAVE_PTHREAD
-#include <pthread.h>
-#endif
 
-#include "tds.h"
+#include <freetds/tds.h>
+#include <freetds/thread.h>
 
-static char software_version[] = "$Id: freeclose.c,v 1.14 2010/09/01 08:39:38 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+/* this crazy test tests that we do not send too much prepare ... */
 
-/* this crazy test test that we do not send too much prepare ... */
+static tds_mutex mtx;
 
-#ifndef _WIN32
-static int
-find_last_socket(void)
-{
-	int max_socket = -1, i;
+typedef union {
+	struct sockaddr sa;
+	struct sockaddr_in sin;
+	char dummy[256];
+} long_sockaddr;
 
-	for (i = 3; i < 1024; ++i) {
-		struct stat file_stat;
-		if (fstat(i, &file_stat))
-			continue;
-		if ((file_stat.st_mode & S_IFSOCK) == S_IFSOCK)
-			max_socket = i;
-	}
-	return max_socket;
-}
-#else
-static TDS_SYS_SOCKET
-find_last_socket(void)
-{
-	TDS_SYS_SOCKET max_socket = INVALID_SOCKET;
-	int i;
-
-	for (i = 4; i <= (4096*4); i += 4) {
-		struct sockaddr addr;
-		socklen_t addr_len;
-
-		if (tds_getpeername((TDS_SYS_SOCKET) i, &addr, &addr_len))
-			continue;
-		max_socket = (TDS_SYS_SOCKET) i;
-	}
-	
-	return max_socket;
-}
-#endif
-
-static struct sockaddr remote_addr;
+static long_sockaddr remote_addr;
 static socklen_t remote_addr_len;
 
 static TDS_SYS_SOCKET fake_sock;
 
-#ifndef _WIN32
-static pthread_t      fake_thread;
-#define THREADAPI
-#define THREADRET void*
-#else
-static HANDLE fake_thread;
-#define THREADAPI WINAPI
-#define THREADRET DWORD
-#define pthread_join(th,fl) WaitForSingleObject(th,INFINITE)
+static tds_thread fake_thread;
+#ifdef _WIN32
 #define alarm(n) do { ; } while(0)
 #endif
-static THREADRET THREADAPI fake_thread_proc(void *arg);
+static TDS_THREAD_PROC_DECLARE(fake_thread_proc, arg);
 
 static int
 init_fake_server(int ip_port)
@@ -117,7 +65,7 @@ init_fake_server(int ip_port)
 	TDS_SYS_SOCKET s;
 
 	memset(&sin, 0, sizeof(sin));
-	sin.sin_addr.s_addr = INADDR_ANY;
+	sin.sin_addr.s_addr = htonl(INADDR_LOOPBACK);
 	sin.sin_port = htons((short) ip_port);
 	sin.sin_family = AF_INET;
 
@@ -130,19 +78,15 @@ init_fake_server(int ip_port)
 		CLOSESOCKET(s);
 		return 1;
 	}
-	listen(s, 5);
-#ifndef _WIN32
-	if (pthread_create(&fake_thread, NULL, fake_thread_proc, int2ptr(s)) != 0) {
-		perror("pthread_create");
+	if (listen(s, 5) < 0) {
+		perror("listen");
+		CLOSESOCKET(s);
+		return 1;
+	}
+	if (tds_thread_create(&fake_thread, fake_thread_proc, int2ptr(s)) != 0) {
+		perror("tds_thread_create");
 		exit(1);
 	}
-#else
-	fake_thread = CreateThread(NULL, 0, fake_thread_proc, int2ptr(s), 0, NULL);
-	if (!fake_thread) {
-		fprintf(stderr, "CreateThread error %u\n", (unsigned) GetLastError());
-		exit(1);
-	}
-#endif
 	return 0;
 }
 
@@ -197,7 +141,9 @@ count_insert(const char* buf, size_t len)
 
 	/* check it */
 	while ((p=strstr(insert_buf, search)) != NULL) {
+		tds_mutex_lock(&mtx);
 		++inserts;
+		tds_mutex_unlock(&mtx);
 		/* do not find again */
 		p[0] = '_';
 	}
@@ -213,32 +159,38 @@ count_insert(const char* buf, size_t len)
 static unsigned int round_trips = 0;
 static enum { sending, receiving } flow = sending;
 
-static THREADRET THREADAPI
-fake_thread_proc(void * arg)
+static TDS_THREAD_PROC_DECLARE(fake_thread_proc, arg)
 {
 	TDS_SYS_SOCKET s = ptr2int(arg), server_sock;
-	socklen_t len;
+	socklen_t sock_len;
+	int len;
 	char buf[128];
 	struct sockaddr_in sin;
 	fd_set fds_read, fds_write, fds_error;
-	int max_fd = 0;
+	TDS_SYS_SOCKET max_fd = 0;
 
 	memset(&sin, 0, sizeof(sin));
-	len = sizeof(sin);
+	sock_len = sizeof(sin);
 	alarm(30);
-	if ((fake_sock = tds_accept(s, (struct sockaddr *) &sin, &len)) < 0) {
+	fprintf(stderr, "waiting connect...\n");
+	if ((fake_sock = tds_accept(s, (struct sockaddr *) &sin, &sock_len)) < 0) {
 		perror("accept");
 		exit(1);
 	}
 	CLOSESOCKET(s);
 
-	if (TDS_IS_SOCKET_INVALID(server_sock = socket(AF_INET, SOCK_STREAM, 0))) {
+	if (TDS_IS_SOCKET_INVALID(server_sock = socket(remote_addr.sa.sa_family, SOCK_STREAM, 0))) {
 		perror("socket");
 		exit(1);
 	}
 
-	if (connect(server_sock, &remote_addr, remote_addr_len)) {
-		perror("socket");
+	fprintf(stderr, "connecting to server...\n");
+	if (remote_addr.sa.sa_family == AF_INET) {
+		fprintf(stderr, "connecting to %x:%d\n", (unsigned int) remote_addr.sin.sin_addr.s_addr,
+			ntohs(remote_addr.sin.sin_port));
+	}
+	if (connect(server_sock, &remote_addr.sa, remote_addr_len)) {
+		perror("connect");
 		exit(1);
 	}
 	alarm(0);
@@ -276,35 +228,49 @@ fake_thread_proc(void * arg)
 
 		/* just read and forward */
 		if (FD_ISSET(fake_sock, &fds_read)) {
-			if (flow != sending)
+			if (flow != sending) {
+				tds_mutex_lock(&mtx);
 				++round_trips;
+				tds_mutex_unlock(&mtx);
+			}
 			flow = sending;
 
 			len = READSOCKET(fake_sock, buf, sizeof(buf));
-			if (len == 0)
+			if (len == 0) {
+				fprintf(stderr, "client connection closed\n");
 				break;
-			if (len < 0 && sock_errno != TDSSOCK_EINPROGRESS)
+			}
+			if (len < 0 && sock_errno != TDSSOCK_EINPROGRESS) {
+				fprintf(stderr, "read client error %d\n", sock_errno);
 				break;
+			}
 			count_insert(buf, len);
 			write_all(server_sock, buf, len);
 		}
 
 		if (FD_ISSET(server_sock, &fds_read)) {
-			if (flow != receiving)
+			if (flow != receiving) {
+				tds_mutex_lock(&mtx);
 				++round_trips;
+				tds_mutex_unlock(&mtx);
+			}
 			flow = receiving;
 
 			len = READSOCKET(server_sock, buf, sizeof(buf));
-			if (len == 0)
+			if (len == 0) {
+				fprintf(stderr, "server connection closed\n");
 				break;
-			if (len < 0 && sock_errno != TDSSOCK_EINPROGRESS)
+			}
+			if (len < 0 && sock_errno != TDSSOCK_EINPROGRESS) {
+				fprintf(stderr, "read server error %d\n", sock_errno);
 				break;
+			}
 			write_all(fake_sock, buf, len);
 		}
 	}
 	CLOSESOCKET(fake_sock);
 	CLOSESOCKET(server_sock);
-	return (THREADRET) 0;
+	return TDS_THREAD_RESULT(0);
 }
 
 int
@@ -321,20 +287,29 @@ main(int argc, char **argv)
 
 #ifdef _WIN32
 	WSADATA wsaData;
-	WSAStartup(MAKEWORD(1, 1), &wsaData);
+	WSAStartup(MAKEWORD(2, 2), &wsaData);
 #endif
 
+	if (tds_mutex_init(&mtx))
+		return 1;
+
+	odbc_mark_sockets_opened();
+
 	odbc_connect();
 
-	last_socket = find_last_socket();
+	/*
+	 * this does not work if server is not connected with socket
+	 * (ie ms driver connected locally)
+	 */
+	last_socket = odbc_find_last_socket();
 	if (TDS_IS_SOCKET_INVALID(last_socket)) {
 		fprintf(stderr, "Error finding last socket opened\n");
 		return 1;
 	}
 
 	remote_addr_len = sizeof(remote_addr);
-	if (tds_getpeername(last_socket, &remote_addr, &remote_addr_len)) {
-		fprintf(stderr, "Unable to get remote address\n");
+	if (tds_getpeername(last_socket, &remote_addr.sa, &remote_addr_len)) {
+		fprintf(stderr, "Unable to get remote address %d\n", sock_errno);
 		return 1;
 	}
 
@@ -349,7 +324,7 @@ main(int argc, char **argv)
 		fprintf(stderr, "Cannot bind to a port\n");
 		return 1;
 	}
-	printf("Fake server binded at port %d\n", port);
+	printf("Fake server bound at port %d\n", port);
 
 	/* override connections */
 	if (is_freetds) {
@@ -366,7 +341,7 @@ main(int argc, char **argv)
 		CHKAllocConnect(&odbc_conn, "S");
 		sprintf(tmp, "DRIVER={SQL Server};SERVER=127.0.0.1,%d;UID=%s;PWD=%s;DATABASE=%s;Network=DBMSSOCN;", port, odbc_user, odbc_password, odbc_database);
 		printf("connection string: %s\n", tmp);
-		CHKDriverConnect(NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT, "SI");
+		CHKDriverConnect(NULL, T(tmp), SQL_NTS, (SQLTCHAR *) tmp, sizeof(tmp)/sizeof(SQLTCHAR), &len, SQL_DRIVER_NOPROMPT, "SI");
 		CHKAllocStmt(&odbc_stmt, "S");
 	}
 
@@ -376,16 +351,20 @@ main(int argc, char **argv)
 	odbc_reset_statement();
 
 	/* do not take into account connection statistics */
+	tds_mutex_lock(&mtx);
 	round_trips = 0;
 	inserts = 0;
+	tds_mutex_unlock(&mtx);
 
 	query = "insert into #test values (?, ?)";
 
 	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_SLONG, SQL_INTEGER, sizeof(id), 0, &id, 0, &sql_nts, "SI");
 	CHKBindParameter(2, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_VARCHAR, sizeof(string), 0, string, 0, &sql_nts, "SI");
 
-	CHKPrepare((SQLCHAR *) query, SQL_NTS, "SI");
+	CHKPrepare(T(query), SQL_NTS, "SI");
+	tds_mutex_lock(&mtx);
 	printf("%u round trips %u inserts\n", round_trips, inserts);
+	tds_mutex_unlock(&mtx);
 
 	for (id = 0; id < num_inserts; id++) {
 		sprintf(string, "This is a test (%d)", (int) id);
@@ -393,25 +372,34 @@ main(int argc, char **argv)
 		CHKFreeStmt(SQL_CLOSE, "S");
 	}
 
+	tds_mutex_lock(&mtx);
 	printf("%u round trips %u inserts\n", round_trips, inserts);
+	tds_mutex_unlock(&mtx);
 	odbc_reset_statement();
 
-	if (inserts > 1 || round_trips > num_inserts * 2 + 6) {
+	tds_mutex_lock(&mtx);
+	if (inserts > 1 || round_trips > (unsigned) (num_inserts * 2 + 6)) {
 		fprintf(stderr, "Too much round trips (%u) or insert (%u) !!!\n", round_trips, inserts);
+		tds_mutex_unlock(&mtx);
 		return 1;
 	}
 	printf("%u round trips %u inserts\n", round_trips, inserts);
+	tds_mutex_unlock(&mtx);
 
 #ifdef ENABLE_DEVELOPING
 	/* check for SQL_RESET_PARAMS */
+	tds_mutex_lock(&mtx);
 	round_trips = 0;
 	inserts = 0;
+	tds_mutex_unlock(&mtx);
 
 	CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_SLONG, SQL_INTEGER, sizeof(id), 0, &id, 0, &sql_nts, "SI");
 	CHKBindParameter(2, SQL_PARAM_INPUT, SQL_C_CHAR, SQL_VARCHAR, sizeof(string), 0, string, 0, &sql_nts, "SI");
 
 	CHKPrepare((SQLCHAR *) query, SQL_NTS, "SI");
+	tds_mutex_lock(&mtx);
 	printf("%u round trips %u inserts\n", round_trips, inserts);
+	tds_mutex_unlock(&mtx);
 
 	for (id = 0; id < num_inserts; id++) {
 		CHKBindParameter(1, SQL_PARAM_INPUT, SQL_C_SLONG, SQL_INTEGER, sizeof(id), 0, &id, 0, &sql_nts, "SI");
@@ -422,20 +410,25 @@ main(int argc, char **argv)
 		CHKFreeStmt(SQL_RESET_PARAMS, "S");
 	}
 
+	tds_mutex_lock(&mtx);
 	printf("%u round trips %u inserts\n", round_trips, inserts);
+	tds_mutex_unlock(&mtx);
 	odbc_reset_statement();
 
+	tds_mutex_lock(&mtx);
 	if (inserts > 1 || round_trips > num_inserts * 2 + 6) {
 		fprintf(stderr, "Too much round trips (%u) or insert (%u) !!!\n", round_trips, inserts);
+		tds_mutex_unlock(&mtx);
 		return 1;
 	}
 	printf("%u round trips %u inserts\n", round_trips, inserts);
+	tds_mutex_unlock(&mtx);
 #endif
 
 	odbc_disconnect();
 
 	alarm(10);
-	pthread_join(fake_thread, NULL);
+	tds_thread_join(fake_thread, NULL);
 
 	return 0;
 }
@@ -444,8 +437,9 @@ main(int argc, char **argv)
 int
 main(void)
 {
-        printf("Not possible for this platform.\n");
-        return 0;
+	printf("Not possible for this platform.\n");
+	odbc_test_skipped();
+	return 0;
 }
 #endif
 
