@@ -20,44 +20,40 @@
 #include <sys/wait.h>
 #endif /* HAVE_SYS_WAIT_H */
 
+#include <freetds/utils.h>
+
 /*
  * test error on connection close 
  * With a trick we simulate a connection close then we try to 
  * prepare or execute a query. This should fail and return an error message.
  */
 
-#if HAVE_FSTAT && defined(S_IFSOCK)
+#ifdef _WIN32
+#define SHUT_RDWR SD_BOTH
+#endif
 
 static int
 close_last_socket(void)
 {
 	TDS_SYS_SOCKET max_socket = odbc_find_last_socket();
-	TDS_SYS_SOCKET sockets[2];
 
 	if (max_socket < 0) {
 		fprintf(stderr, "Error finding last socket\n");
 		return 0;
 	}
 
-	/* replace socket with a new one */
-	if (socketpair(AF_UNIX, SOCK_STREAM, 0, sockets) < 0)
-		return 0;
-
-	/* substitute socket */
-	close(max_socket);
-	dup2(sockets[0], max_socket);
-
 	/* close connection */
-	close(sockets[0]);
-	close(sockets[1]);
+	shutdown(max_socket, SHUT_RDWR);
+
 	return 1;
 }
 
 static int
-Test(int direct)
+Test(int direct, int long_query)
 {
 	SQLTCHAR buf[256];
 	SQLTCHAR sqlstate[6];
+	char sql[5100];
 
 	odbc_mark_sockets_opened();
 	odbc_connect();
@@ -67,13 +63,20 @@ Test(int direct)
 		return 1;
 	}
 
+	if (long_query) {
+		memset(sql, '-', sizeof(sql));
+		strcpy(sql + 5000, "\nSELECT 1");
+	} else {
+		strcpy(sql, "SELECT 1");
+	}
+
 	/* force disconnection closing socket */
 	if (direct) {
-		CHKExecDirect(T("SELECT 1"), SQL_NTS, "E");
+		CHKExecDirect(T(sql), SQL_NTS, "E");
 	} else {
 		SQLSMALLINT cols;
 		/* use prepare, force dialog with server */
-		if (CHKPrepare(T("SELECT 1"), SQL_NTS, "SE") == SQL_SUCCESS)
+		if (CHKPrepare(T(sql), SQL_NTS, "SE") == SQL_SUCCESS)
 			CHKNumResultCols(&cols, "E");
 	}
 
@@ -90,18 +93,12 @@ Test(int direct)
 int
 main(void)
 {
-	if (Test(0) || Test(1))
+	/* check with short queries */
+	if (Test(0, 0) || Test(1, 0))
+		return 1;
+
+	/* check with large queries, this will trigger different paths */
+	if (Test(0, 1) || Test(1, 1))
 		return 1;
 	return 0;
 }
-
-#else
-int
-main(void)
-{
-	printf("Not possible for this platform.\n");
-	odbc_test_skipped();
-	return 0;
-}
-#endif
-
