@@ -1,24 +1,14 @@
-#include "common.h"
+/*
+ * Test connection timeout
+ */
 
-/* TODO port to windows, use thread */
-#if defined(TDS_HAVE_PTHREAD_MUTEX) && HAVE_ALARM
+#include "common.h"
 
 #if HAVE_UNISTD_H
 #include <unistd.h>
 #endif /* HAVE_UNISTD_H */
 
-#if TIME_WITH_SYS_TIME
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# endif
-# include <time.h>
-#else
-# if HAVE_SYS_TIME_H
-#  include <sys/time.h>
-# else
-#  include <time.h>
-# endif
-#endif
+#include <freetds/time.h>
 
 #if HAVE_ERRNO_H
 #include <errno.h>
@@ -32,16 +22,15 @@
 #include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
 
-#include <pthread.h>
+#if HAVE_POLL_H
+#include <poll.h>
+#endif /* HAVE_POLL_H */
 
-#include "tds.h"
+#include <freetds/tds.h>
+#include <freetds/thread.h>
+#include "replacements.h"
 
-/*
-	test connection timeout
-*/
-
-static char software_version[] = "$Id: timeout3.c,v 1.12 2010/07/05 09:20:33 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
+#if TDS_HAVE_MUTEX
 
 static void init_connect(void);
 
@@ -53,11 +42,13 @@ init_connect(void)
 	CHKAllocConnect(&odbc_conn, "S");
 }
 
-static pthread_t      fake_thread;
+static tds_thread fake_thread;
+static tds_mutex mtx;
 static TDS_SYS_SOCKET fake_sock;
 
-static void *fake_thread_proc(void * arg);
+static TDS_THREAD_PROC_DECLARE(fake_thread_proc, arg);
 
+/* build a listening socket to connect to */
 static int
 init_fake_server(int ip_port)
 {
@@ -80,58 +71,89 @@ init_fake_server(int ip_port)
 		return 1;
 	}
 	listen(s, 5);
-	err = pthread_create(&fake_thread, NULL, fake_thread_proc, int2ptr(s));
+	err = tds_thread_create(&fake_thread, fake_thread_proc, int2ptr(s));
 	if (err != 0) {
-		perror("pthread_create");
+		perror("tds_thread_create");
 		exit(1);
 	}
 	return 0;
 }
 
-static void *
-fake_thread_proc(void * arg)
+/* accept a socket and read data as much as you can */
+static TDS_THREAD_PROC_DECLARE(fake_thread_proc, arg)
 {
-	TDS_SYS_SOCKET s = ptr2int(arg);
+	TDS_SYS_SOCKET s = ptr2int(arg), sock;
 	socklen_t len;
 	char buf[128];
 	struct sockaddr_in sin;
+	struct pollfd fd;
 
 	memset(&sin, 0, sizeof(sin));
 	len = sizeof(sin);
-	alarm(30);
-	if (TDS_IS_SOCKET_INVALID(fake_sock = tds_accept(s, (struct sockaddr *) &sin, &len))) {
+
+	fd.fd = s;
+	fd.events = POLLIN;
+	fd.revents = 0;
+	if (poll(&fd, 1, 30000) <= 0) {
+		fprintf(stderr, "poll: %d\n", sock_errno);
+		exit(1);
+	}
+
+	if (TDS_IS_SOCKET_INVALID(sock = tds_accept(s, (struct sockaddr *) &sin, &len))) {
 		perror("accept");
 		exit(1);
 	}
+	tds_mutex_lock(&mtx);
+	fake_sock = sock;
+	tds_mutex_unlock(&mtx);
 	CLOSESOCKET(s);
 
 	for (;;) {
+		int len;
+
+		fd.fd = sock;
+		fd.events = POLLIN;
+		fd.revents = 0;
+		if (poll(&fd, 1, 30000) <= 0) {
+			fprintf(stderr, "poll: %d\n", sock_errno);
+			exit(1);
+		}
+
 		/* just read and discard */
-		len = READSOCKET(fake_sock, buf, sizeof(buf));
+		len = READSOCKET(sock, buf, sizeof(buf));
 		if (len == 0)
 			break;
 		if (len < 0 && sock_errno != TDSSOCK_EINPROGRESS)
 			break;
 	}
-	return NULL;
+	return TDS_THREAD_RESULT(0);
 }
 
 int
 main(int argc, char *argv[])
 {
-	char tmp[2048];
-	char sqlstate[6];
+	SQLTCHAR tmp[2048];
+	char conn[128];
+	SQLTCHAR sqlstate[6];
 	SQLSMALLINT len;
 	int port;
 	time_t start_time, end_time;
 
+#ifdef _WIN32
+	WSADATA wsaData;
+	WSAStartup(MAKEWORD(2, 2), &wsaData);
+#endif
+
+	if (tds_mutex_init(&mtx))
+		return 1;
+
 	if (odbc_read_login_info())
 		exit(1);
 
 	/*
 	 * prepare our odbcinst.ini 
-	 * is better to do it before connect cause uniODBC cache INIs
-	 * the name must be odbcinst.ini cause unixODBC accept only this name
+	 * it is better to do it before connecting because unixODBC caches INIs
+	 * the name must be odbcinst.ini because unixODBC accepts only this name
 	 */
 	if (odbc_driver[0]) {
 		FILE *f = fopen("odbcinst.ini", "w");
@@ -147,6 +169,9 @@ main(int argc, char *argv[])
 		}
 	}
 
+	/* this test requires version 7.0, avoid to override externally */
+	setenv("TDSVER", "7.0", 1);
+
 	for (port = 12340; port < 12350; ++port)
 		if (!init_fake_server(port))
 			break;
@@ -154,28 +179,30 @@ main(int argc, char *argv[])
 		fprintf(stderr, "Cannot bind to a port\n");
 		return 1;
 	}
-	printf("Fake server binded at port %d\n", port);
+	printf("Fake server bound at port %d\n", port);
 
 	init_connect();
-	CHKSetConnectAttr(SQL_ATTR_CONNECTION_TIMEOUT, (SQLPOINTER) 10, sizeof(SQLINTEGER), "SI");
-	CHKSetConnectAttr(SQL_ATTR_LOGIN_TIMEOUT, (SQLPOINTER) 10, sizeof(SQLINTEGER), "SI");
+	CHKSetConnectAttr(SQL_ATTR_CONNECTION_TIMEOUT, (SQLPOINTER) 25, sizeof(SQLINTEGER), "SI");
+	CHKSetConnectAttr(SQL_ATTR_LOGIN_TIMEOUT, (SQLPOINTER) 10, SQL_IS_UINTEGER, "SI");
 
 	/* this is expected to work with unixODBC */
 	printf("try to connect to our port just to check connection timeout\n");
-	sprintf(tmp, "DRIVER=FreeTDS;SERVER=127.0.0.1;Port=%d;TDS_Version=7.0;UID=test;PWD=test;DATABASE=tempdb;", port);
+	sprintf(conn, "DRIVER=FreeTDS;SERVER=127.0.0.1;Port=%d;TDS_Version=7.0;UID=test;PWD=test;DATABASE=tempdb;", port);
 	start_time = time(NULL);
-	CHKDriverConnect(NULL, (SQLCHAR *) tmp, SQL_NTS, (SQLCHAR *) tmp, sizeof(tmp), &len, SQL_DRIVER_NOPROMPT, "E");
+	CHKDriverConnect(NULL, T(conn), SQL_NTS, tmp, ODBC_VECTOR_SIZE(tmp), &len, SQL_DRIVER_NOPROMPT, "E");
 	end_time = time(NULL);
 
-	strcpy(sqlstate, "XXXXX");
+	memset(sqlstate, 'X', sizeof(sqlstate));
 	tmp[0] = 0;
-	CHKGetDiagRec(SQL_HANDLE_DBC, odbc_conn, 1, (SQLCHAR *) sqlstate, NULL, (SQLCHAR *) tmp, sizeof(tmp), NULL, "SI");
+	CHKGetDiagRec(SQL_HANDLE_DBC, odbc_conn, 1, sqlstate, NULL, tmp, ODBC_VECTOR_SIZE(tmp), NULL, "SI");
 	odbc_disconnect();
+	tds_mutex_lock(&mtx);
 	CLOSESOCKET(fake_sock);
-	pthread_join(fake_thread, NULL);
+	tds_mutex_unlock(&mtx);
+	tds_thread_join(fake_thread, NULL);
 
-	printf("Message: %s - %s\n", sqlstate, tmp);
-	if (strcmp(sqlstate, "HYT00") || !strstr(tmp, "Timeout")) {
+	printf("Message: %s - %s\n", C(sqlstate), C(tmp));
+	if (strcmp(C(sqlstate), "HYT00") || !strstr(C(tmp), "Timeout")) {
 		fprintf(stderr, "Invalid timeout message\n");
 		return 1;
 	}
@@ -185,14 +212,16 @@ main(int argc, char *argv[])
 	}
 
 	printf("Done.\n");
+	ODBC_FREE();
 	return 0;
 }
 
-#else	/* !TDS_HAVE_PTHREAD_MUTEX */
+#else	/* !TDS_HAVE_MUTEX */
 int
 main(void)
 {
 	printf("Not possible for this platform.\n");
+	odbc_test_skipped();
 	return 0;
 }
 #endif
