@@ -22,6 +22,8 @@
 #include <sys/wait.h>
 #endif /* HAVE_SYS_WAIT_H */
 
+#include <freetds/utils.h>
+
 /*
  * test error on connection close 
  * With a trick we simulate a connection close then we try to 
@@ -45,6 +47,8 @@ shutdown_last_socket(void)
 	if (socketpair(AF_UNIX, SOCK_STREAM, 0, sockets) < 0)
 		return 0;
 
+	tds_socket_set_nosigpipe(sockets[0], 1);
+
 	/* substitute socket */
 	close(max_socket);
 	dup2(sockets[0], max_socket);
