@@ -32,7 +32,7 @@ static const struct type types[] = {
 /*	TYPE_C(SQL_C_TIMESTAMP), */
 	TYPE_BOTH(SQL_C_TIMESTAMP, SQL_TIMESTAMP),
 	TYPE_BOTH(SQL_C_TYPE_DATE, SQL_TYPE_DATE),
-	TYPE_C(SQL_C_TYPE_TIME),
+	TYPE_BOTH(SQL_C_TYPE_TIME, SQL_TYPE_TIME),
 	TYPE_BOTH(SQL_C_TYPE_TIMESTAMP, SQL_TYPE_TIMESTAMP),
 	TYPE_C(SQL_C_INTERVAL_YEAR),
 	TYPE_C(SQL_C_INTERVAL_MONTH),
