@@ -27,10 +27,9 @@
 
 #include <stdarg.h>
 #include <stdio.h>
-#include <unistd.h>
 #include <signal.h>
-#include <fcntl.h>
 #include <errno.h>
+#include <fcntl.h>
 
 #if HAVE_STDLIB_H
 #include <stdlib.h>
@@ -48,6 +47,10 @@
 #include <sys/socket.h>
 #endif /* HAVE_SYS_SOCKET_H */
 
+#if HAVE_POLL_H
+#include <poll.h>
+#endif /* HAVE_POLL_H */
+
 #if HAVE_NETINET_IN_H
 #include <netinet/in.h>
 #endif /* HAVE_NETINET_IN_H */
@@ -56,16 +59,20 @@
 #include <arpa/inet.h>
 #endif /* HAVE_ARPA_INET_H */
 
+#ifdef _WIN32
+#include <io.h>
+#endif
+
 #include "pool.h"
+#include "replacements.h"
 
 /* to be set by sig term */
 static int got_sigterm = 0;
-static int got_sighup = 0;
 static const char *logfile_name = NULL;
 
 static void sigterm_handler(int sig);
 static void pool_schedule_waiters(TDS_POOL * pool);
-static TDS_POOL *pool_init(const char *name);
+static TDS_POOL *pool_init(const char *name, const char *config_path);
 static void pool_socket_init(TDS_POOL * pool);
 static void pool_main_loop(TDS_POOL * pool);
 static bool pool_open_logfile(TDS_POOL * pool);
@@ -76,11 +83,15 @@ sigterm_handler(int sig)
 	got_sigterm = 1;
 }
 
+#ifndef _WIN32
+static int got_sighup = 0;
+
 static void
 sighup_handler(int sig)
 {
 	got_sighup = 1;
 }
+#endif
 
 static void
 check_field(const char *pool_name, bool cond, const char *field_name)
@@ -95,7 +106,7 @@ check_field(const char *pool_name, bool cond, const char *field_name)
  * pool_init creates a named pool and opens connections to the database
  */
 static TDS_POOL *
-pool_init(const char *name)
+pool_init(const char *name, const char *config_path)
 {
 	TDS_POOL *pool;
 	char *err = NULL;
@@ -112,7 +123,7 @@ pool_init(const char *name)
 	}
 
 	/* FIXME -- read this from the conf file */
-	if (!pool_read_conf_file(name, pool, &err)) {
+	if (!pool_read_conf_files(config_path, name, pool, &err)) {
 		fprintf(stderr, "Configuration for pool ``%s'' not found.\n", name);
 		exit(EXIT_FAILURE);
 	}
@@ -125,6 +136,11 @@ pool_init(const char *name)
 	check_field(name, pool->server != NULL, "server");
 	check_field(name, pool->port != 0,   "port");
 
+	if (!pool->server_user)
+		pool->server_user = strdup(pool->user);
+	if (!pool->server_password)
+		pool->server_password = strdup(pool->password);
+
 	if (pool->max_open_conn < pool->min_open_conn) {
 		fprintf(stderr, "Max connections less than minimum\n");
 		exit(EXIT_FAILURE);
@@ -158,6 +174,8 @@ pool_destroy(TDS_POOL *pool)
 	free(pool->server);
 	free(pool->database);
 	free(pool->name);
+	free(pool->server_user);
+	free(pool->server_password);
 	free(pool);
 }
 
@@ -330,10 +348,12 @@ pool_main_loop(TDS_POOL * pool)
 		if (TDS_UNLIKELY(got_sigterm))
 			break;
 
+#ifndef _WIN32
 		if (TDS_UNLIKELY(got_sighup)) {
 			got_sighup = 0;
 			pool_open_logfile(pool);
 		}
+#endif
 
 		/* process events */
 		if (FD_ISSET(wakeup, &sel.rfds)) {
@@ -360,7 +380,7 @@ pool_main_loop(TDS_POOL * pool)
 static void
 print_usage(const char *progname)
 {
-	fprintf(stderr, "Usage:\t%s [-l <log file>] [-d] <pool name>\n", progname);
+	fprintf(stderr, "Usage:\t%s [-l <log file>] [-c <conf file>] [-d] <pool name>\n", progname);
 }
 
 int
@@ -374,6 +394,7 @@ main(int argc, char **argv)
 #  define DAEMON_OPT ""
 #endif
 	TDS_POOL *pool;
+	const char *config_path = NULL;
 
 	signal(SIGTERM, sigterm_handler);
 	signal(SIGINT, sigterm_handler);
@@ -382,7 +403,7 @@ main(int argc, char **argv)
 	signal(SIGPIPE, SIG_IGN);
 #endif
 
-	while ((opt = getopt(argc, argv, "l:" DAEMON_OPT)) != -1) {
+	while ((opt = getopt(argc, argv, "l:c:" DAEMON_OPT)) != -1) {
 		switch (opt) {
 		case 'l':
 			logfile_name = optarg;
@@ -392,6 +413,9 @@ main(int argc, char **argv)
 			daemonize = true;
 			break;
 #endif
+		case 'c':
+			config_path = optarg;
+			break;
 		default:
 			print_usage(argv[0]);
 			return EXIT_FAILURE;
@@ -401,7 +425,7 @@ main(int argc, char **argv)
 		print_usage(argv[0]);
 		return EXIT_FAILURE;
 	}
-	pool = pool_init(argv[optind]);
+	pool = pool_init(argv[optind], config_path);
 #ifdef HAVE_FORK
 	if (daemonize) {
 		if (daemon(0, 0) < 0) {
