@@ -53,7 +53,7 @@
 
 #include "pool.h"
 #include "replacements.h"
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 
 #ifndef MAXHOSTNAMELEN
 #define MAXHOSTNAMELEN 256
@@ -86,8 +86,8 @@ pool_mbr_login(const TDS_POOL * pool, int tds_version)
 	login = tds_alloc_login(1);
 	if (gethostname(hostname, MAXHOSTNAMELEN) < 0)
 		strlcpy(hostname, "tdspool", MAXHOSTNAMELEN);
-	if (!tds_set_passwd(login, pool->password)
-	    || !tds_set_user(login, pool->user)
+	if (!tds_set_passwd(login, pool->server_password)
+	    || !tds_set_user(login, pool->server_user)
 	    || !tds_set_app(login, "tdspool")
 	    || !tds_set_host(login, hostname)
 	    || !tds_set_library(login, "TDS-Library")
@@ -332,7 +332,7 @@ pool_process_members(TDS_POOL * pool, fd_set * rfds, fd_set * wfds)
 {
 	TDS_POOL_MEMBER *pmbr, *next;
 	TDSSOCKET *tds;
-	int age;
+	time_t age;
 	time_t time_now;
 
 	for (next = dlist_member_first(&pool->active_members); (pmbr = next) != NULL; ) {
@@ -375,7 +375,7 @@ pool_process_members(TDS_POOL * pool, fd_set * rfds, fd_set * wfds)
 		age = time_now - pmbr->last_used_tm;
 		if (age > pool->max_member_age
 		    && pool->num_active_members > pool->min_open_conn) {
-			tdsdump_log(TDS_DBG_INFO1, "member is %d seconds old...closing\n", age);
+			tdsdump_log(TDS_DBG_INFO1, "member is %ld seconds old...closing\n", (long int) age);
 			pool_free_member(pool, pmbr);
 		}
 	}
@@ -422,12 +422,12 @@ static TDS_THREAD_PROC_DECLARE(connect_proc, arg)
 				break;
 
 		pool_event_add(pool, &ev->common, connect_execute_ok);
-		return NULL;
+		return TDS_THREAD_RESULT(0);
 	}
 
 	/* failure */
 	pool_event_add(pool, &ev->common, connect_execute_ko);
-	return NULL;
+	return TDS_THREAD_RESULT(0);
 }
 
 static void
