@@ -46,7 +46,7 @@
 
 #include "pool.h"
 #include <freetds/server.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 
 static TDS_POOL_USER *pool_user_find_new(TDS_POOL * pool);
 static bool pool_user_login(TDS_POOL * pool, TDS_POOL_USER * puser);
@@ -111,7 +111,7 @@ static TDS_THREAD_PROC_DECLARE(login_proc, arg)
 	ev->success = pool_user_login(ev->pool, ev->puser);
 
 	pool_event_add(ev->pool, &ev->common, login_execute);
-	return NULL;
+	return TDS_THREAD_RESULT(0);
 }
 
 static void
@@ -453,36 +453,37 @@ pool_user_read(TDS_POOL * pool, TDS_POOL_USER * puser)
 	TDS_POOL_MEMBER *pmbr = NULL;
 
 	for (;;) {
+		TDS_UCHAR in_flag;
+
 		if (pool_packet_read(tds))
 			break;
 		if (tds->in_len == 0) {
 			tdsdump_log(TDS_DBG_INFO1, "user disconnected\n");
 			pool_free_user(pool, puser);
 			return false;
-		} else {
-			TDS_UCHAR in_flag = tds->in_buf[0];
+		}
 
-			tdsdump_dump_buf(TDS_DBG_NETWORK, "Got packet from client:", tds->in_buf, tds->in_len);
+		tdsdump_dump_buf(TDS_DBG_NETWORK, "Got packet from client:", tds->in_buf, tds->in_len);
 
-			switch (in_flag) {
-			case TDS_QUERY:
-			case TDS_NORMAL:
-			case TDS_RPC:
-			case TDS_BULK:
-			case TDS_CANCEL:
-			case TDS7_TRANS:
-				if (!pool_write_data(&puser->sock, &puser->assigned_member->sock)) {
-					pool_reset_member(pool, puser->assigned_member);
-					return false;
-				}
-				pmbr = puser->assigned_member;
-				break;
-
-			default:
-				tdsdump_log(TDS_DBG_ERROR, "Unrecognized packet type, closing user\n");
-				pool_free_user(pool, puser);
+		in_flag = tds->in_buf[0];
+		switch (in_flag) {
+		case TDS_QUERY:
+		case TDS_NORMAL:
+		case TDS_RPC:
+		case TDS_BULK:
+		case TDS_CANCEL:
+		case TDS7_TRANS:
+			if (!pool_write_data(&puser->sock, &puser->assigned_member->sock)) {
+				pool_reset_member(pool, puser->assigned_member);
 				return false;
 			}
+			pmbr = puser->assigned_member;
+			break;
+
+		default:
+			tdsdump_log(TDS_DBG_ERROR, "Unrecognized packet type, closing user\n");
+			pool_free_user(pool, puser);
+			return false;
 		}
 		if (tds->in_pos < tds->in_len)
 			/* partial write, schedule a future write */
@@ -534,7 +535,7 @@ static TDS_THREAD_PROC_DECLARE(end_login_proc, arg)
 	ev->success = pool_user_send_login_ack(pool, ev->puser);
 
 	pool_event_add(pool, &ev->common, end_login_execute);
-	return NULL;
+	return TDS_THREAD_RESULT(0);
 }
 
 static void
