@@ -17,30 +17,28 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #if HAVE_STRING_H
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include "tds_sysdep_private.h"
+#include <freetds/sysdep_private.h>
 #include "replacements.h"
 
-/* $Id: strlcpy.c,v 1.1 2005/07/18 18:50:20 freddy77 Exp $ */
-
 size_t 
 tds_strlcpy(char *dest, const char *src, size_t len)
 {
 	size_t l = strlen(src);
 
-	--len;
-	if (l > len) {
-		memcpy(dest, src, len);
-		dest[len] = 0;
-	} else {
-		memcpy(dest, src, l + 1);
+	if (len) {
+		--len;
+		if (l > len) {
+			memcpy(dest, src, len);
+			dest[len] = 0;
+		} else {
+			memcpy(dest, src, l + 1);
+		}
 	}
 	return l;
 }
