@@ -35,8 +35,9 @@
 #endif /* HAVE_UNISTD_H */
 
 #include <freetds/tds.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #include <freetds/server.h>
+#include <freetds/utils.h>
 
 static void dump_login(TDSLOGIN * login);
 
