@@ -72,7 +72,8 @@
 
 #include <freetds/tds.h>
 #include <freetds/configs.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
+#include <freetds/utils.h>
 #include "replacements.h"
 
 static int tds_config_login(TDSLOGIN * connection, TDSLOGIN * login);
@@ -80,7 +81,7 @@ static int tds_config_env_tdsdump(TDSLOGIN * login);
 static void tds_config_env_tdsver(TDSLOGIN * login);
 static void tds_config_env_tdsport(TDSLOGIN * login);
 static int tds_config_env_tdshost(TDSLOGIN * login);
-static int tds_read_conf_sections(FILE * in, const char *server, TDSLOGIN * login);
+static bool tds_read_conf_sections(FILE * in, const char *server, TDSLOGIN * login);
 static int tds_read_interfaces(const char *server, TDSLOGIN * login);
 static int parse_server_name_for_port(TDSLOGIN * connection, TDSLOGIN * login);
 static int tds_lookup_port(const char *portname);
@@ -100,7 +101,7 @@ static const char interfaces_path[] = "/etc/freetds";
 #else
        const char STD_DATETIME_FMT[] = "%b %d %Y %I:%M%p"; /* msvcr80.dll does not support %e */
 static const char pid_config_logpath[] = "c:\\tdsconfig.log.%d";
-static const char freetds_conf [] = "%s\\freetds.conf";
+static const char freetds_conf[] = "%s\\freetds.conf";
 static const char location[] = "(from $FREETDS)";
 static const char pid_logpath[] = "c:\\freetds.log.%d";
 static const char interfaces_path[] = "c:\\";
@@ -140,7 +141,8 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 	char *s;
 	char *path;
 	pid_t pid;
-	int opened = 0, found;
+	int opened = 0;
+	bool found;
 	struct addrinfo *addrs;
 
 	/* allocate a new structure with hard coded and build-time defaults */
@@ -156,7 +158,7 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 			opened = tdsdump_open(s);
 		} else {
 			pid = getpid();
-			if (asprintf(&path, pid_config_logpath, pid) >= 0) {
+			if (asprintf(&path, pid_config_logpath, (int) pid) >= 0) {
 				if (*path) {
 					opened = tdsdump_open(path);
 				}
@@ -183,7 +185,7 @@ tds_read_config_info(TDSSOCKET * tds, TDSLOGIN * login, TDSLOCALE * locale)
 					tds_free_login(connection);
 					return NULL;
 				}
-				found = 1;
+				found = true;
 			}
 		}
 	}
@@ -287,10 +289,10 @@ tds_fix_login(TDSLOGIN * login)
 	tds_config_env_tdshost(login);
 }
 
-static int
+static bool
 tds_try_conf_file(const char *path, const char *how, const char *server, TDSLOGIN * login)
 {
-	int found = 0;
+	bool found = false;
 	FILE *in;
 
 	if ((in = fopen(path, "r")) == NULL) {
@@ -317,7 +319,7 @@ tds_try_conf_file(const char *path, const char *how, const char *server, TDSLOGI
  * Return filename from HOME directory
  * @return allocated string or NULL if error
  */
-static char *
+char *
 tds_get_home_file(const char *file)
 {
 	char *home, *path;
@@ -338,12 +340,12 @@ tds_get_home_file(const char *file)
  * @param server       section of file configuration that hold 
  *                     configuration for a server
  */
-int
+bool
 tds_read_conf_file(TDSLOGIN * login, const char *server)
 {
 	char *path = NULL;
 	char *eptr = NULL;
-	int found = 0;
+	bool found = false;
 
 	if (interf_file) {
 		found = tds_try_conf_file(interf_file, "set programmatically", server, login);
@@ -389,28 +391,28 @@ tds_read_conf_file(TDSLOGIN * login, const char *server)
 	return found;
 }
 
-static int
+static bool
 tds_read_conf_sections(FILE * in, const char *server, TDSLOGIN * login)
 {
 	DSTR default_instance = DSTR_INITIALIZER;
 	int default_port;
 
-	int found;
+	bool found;
 
 	tds_read_conf_section(in, "global", tds_parse_conf_section, login);
 
 	if (!server[0])
-		return 0;
+		return false;
 	rewind(in);
 
 	if (!tds_dstr_dup(&default_instance, &login->instance_name))
-		return 0;
+		return false;
 	default_port = login->port;
 
 	found = tds_read_conf_section(in, server, tds_parse_conf_section, login);
 	if (!login->valid_configuration) {
 		tds_dstr_free(&default_instance);
-		return 0;
+		return false;
 	}
 
 	/* 
@@ -495,7 +497,7 @@ tds_config_encryption(const char * value, TDSLOGIN * login)
  * @param tds_conf_parse callback that receive every entry in section
  * @param param          parameter to pass to callback function
  */
-int
+bool
 tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_parse, void *param)
 {
 	char line[256], *value;
@@ -504,7 +506,7 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 	char p;
 	int i;
 	int insection = 0;
-	int found = 0;
+	bool found = false;
 
 	tdsdump_log(TDS_DBG_INFO1, "Looking for section %s.\n", section);
 	while (fgets(line, sizeof(line), in)) {
@@ -570,7 +572,7 @@ tds_read_conf_section(FILE * in, const char *section, TDSCONFPARSE tds_conf_pars
 			if (!strcasecmp(section, &option[1])) {
 				tdsdump_log(TDS_DBG_INFO1, "Got a match.\n");
 				insection = 1;
-				found = 1;
+				found = true;
 			} else {
 				insection = 0;
 			}
@@ -661,6 +663,7 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		s = tds_dstr_copy(&login->server_name, value);
 	} else if (!strcmp(option, TDS_STR_USENTLMV2)) {
 		login->use_ntlmv2 = tds_config_boolean(option, value, login);
+		login->use_ntlmv2_specified = 1;
 	} else if (!strcmp(option, TDS_STR_USELANMAN)) {
 		login->use_lanman = tds_config_boolean(option, value, login);
 	} else if (!strcmp(option, TDS_STR_REALM)) {
@@ -682,6 +685,8 @@ tds_parse_conf_section(const char *option, const char *value, void *param)
 		tdsdump_log(TDS_DBG_FUNC, "Setting ReadOnly Intent to '%s'.\n", value);
 	} else if (!strcmp(option, TLS_STR_OPENSSL_CIPHERS)) {
 		s = tds_dstr_copy(&login->openssl_ciphers, value);
+	} else if (!strcmp(option, TDS_STR_ENABLE_TLS_V1)) {
+		login->enable_tls_v1 = tds_config_boolean(option, value, login);
 	} else {
 		tdsdump_log(TDS_DBG_INFO1, "UNRECOGNIZED option '%s' ... ignoring.\n", option);
 	}
@@ -773,6 +778,12 @@ tds_config_login(TDSLOGIN * connection, TDSLOGIN * login)
 	if (login->readonly_intent)
 		connection->readonly_intent = login->readonly_intent;
 	connection->use_new_password = login->use_new_password;
+
+	if (login->use_ntlmv2_specified) {
+		connection->use_ntlmv2_specified = login->use_ntlmv2_specified;
+		connection->use_ntlmv2 = login->use_ntlmv2;
+	}
+
 	if (res)
 		res = tds_dstr_dup(&connection->new_password, &login->new_password);
 
@@ -782,27 +793,27 @@ tds_config_login(TDSLOGIN * connection, TDSLOGIN * login)
 static int
 tds_config_env_tdsdump(TDSLOGIN * login)
 {
-	char *s;
-	char *path;
-	pid_t pid = 0;
+	char *s = getenv("TDSDUMP");
+	if (!s)
+		return 1;
 
-	if ((s = getenv("TDSDUMP"))) {
-		if (!strlen(s)) {
-			pid = getpid();
-			if (asprintf(&path, pid_logpath, pid) < 0)
-				return 0;
-			if (!tds_dstr_set(&login->dump_file, path)) {
-				free(path);
-				return 0;
-			}
-		} else {
-			if (!tds_dstr_copy(&login->dump_file, s))
-				return 0;
+	if (!strlen(s)) {
+		char *path;
+		pid_t pid = getpid();
+		if (asprintf(&path, pid_logpath, (int) pid) < 0)
+			return 0;
+		if (!tds_dstr_set(&login->dump_file, path)) {
+			free(path);
+			return 0;
 		}
-		tdsdump_log(TDS_DBG_INFO1, "Setting 'dump_file' to '%s' from $TDSDUMP.\n", tds_dstr_cstr(&login->dump_file));
+	} else {
+		if (!tds_dstr_copy(&login->dump_file, s))
+			return 0;
 	}
+	tdsdump_log(TDS_DBG_INFO1, "Setting 'dump_file' to '%s' from $TDSDUMP.\n", tds_dstr_cstr(&login->dump_file));
 	return 1;
 }
+
 static void
 tds_config_env_tdsport(TDSLOGIN * login)
 {
@@ -1333,14 +1344,8 @@ tds_get_compiletime_settings(void)
 #		else
 			, 0
 #		endif
-#		ifdef TDS42
-			, "4.2"
-#		elif TDS46
-			, "4.6"
-#		elif TDS50
+#		if TDS50
 			, "5.0"
-#		elif TDS70
-			, "7.0"
 #		elif TDS71
 			, "7.1"
 #		elif TDS72
