@@ -93,7 +93,7 @@ static const char *parse_numeric(const char *buf, const char *pend,
 #define IS_UINT(x) (0 <= (x) && (x) <= TDS_UINT_MAX)
 
 #define TDS_INT8_MAX ((((TDS_INT8) 0x7fffffffl) << 32) + (TDS_INT8) 0xfffffffflu)
-#define TDS_INT8_MIN  (((TDS_INT8) (-0x7fffffffl-1)) << 32)
+#define TDS_INT8_MIN (-(((TDS_INT8)1)<<62) -(((TDS_INT8)1)<<62))
 #define IS_INT8(x) (TDS_INT8_MIN <= (x) && (x) <= TDS_INT8_MAX)
 
 #define TDS_UINT8_MAX ((((TDS_UINT8) 0xfffffffflu) << 32) + 0xfffffffflu)
@@ -172,9 +172,6 @@ string_to_result(int desttype, const char *s, CONV_RESULT * cr)
 	return (TDS_INT)len;
 }
 
-#define string_to_result(s, cr) \
-	string_to_result(desttype, s, cr)
-
 /**
  * Copy binary data to to result and return len or TDS_CONVERT_NOMEM
  */
@@ -625,7 +622,7 @@ tds_convert_int(TDS_INT num, int desttype, CONV_RESULT * cr)
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
 		sprintf(tmp_str, "%d", num);
-		return string_to_result(tmp_str, cr);
+		return string_to_result(desttype, tmp_str, cr);
 		break;
 	case SYBINT1:
 	case SYBUINT1:
@@ -723,7 +720,7 @@ tds_convert_int8(const TDS_INT8 *src, int desttype, CONV_RESULT * cr)
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
 		sprintf(tmp_str, "%" PRId64, buf);
-		return string_to_result(tmp_str, cr);
+		return string_to_result(desttype, tmp_str, cr);
 		break;
 	case SYBINT1:
 	case SYBUINT1:
@@ -801,7 +798,7 @@ tds_convert_uint8(const TDS_UINT8 *src, int desttype, CONV_RESULT * cr)
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
 		sprintf(tmp_str, "%" PRIu64, buf);
-		return string_to_result(tmp_str, cr);
+		return string_to_result(desttype, tmp_str, cr);
 		break;
 	case SYBINT1:
 	case SYBUINT1:
@@ -874,7 +871,7 @@ tds_convert_numeric(const TDS_NUMERIC * src, int desttype, CONV_RESULT * cr)
 	case CASE_ALL_CHAR:
 		if (tds_numeric_to_string(src, tmpstr) < 0)
 			return TDS_CONVERT_FAIL;
-		return string_to_result(tmpstr, cr);
+		return string_to_result(desttype, tmpstr, cr);
 		break;
 	case SYBINT1:
 	case SYBUINT1:
@@ -1075,7 +1072,7 @@ tds_convert_money4(const TDSCONTEXT * tds_ctx, const TDS_MONEY4 * src, int destt
 		} else {
 			sprintf(p, "%u.%04u", dollars / 10000u, dollars % 10000u);
 		}
-		return string_to_result(tmp_str, cr);
+		return string_to_result(desttype, tmp_str, cr);
 		} break;
 	case SYBINT1:
 	case SYBUINT1:
@@ -1174,7 +1171,7 @@ tds_convert_money(const TDSCONTEXT * tds_ctx, const TDS_MONEY * src, int desttyp
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
 		s = tds_money_to_string((const TDS_MONEY *) src, tmpstr, tds_ctx->money_use_2_digits);
-		return string_to_result(s, cr);
+		return string_to_result(desttype, s, cr);
 		break;
 	case SYBINT1:
 	case SYBUINT1:
@@ -1274,7 +1271,7 @@ tds_convert_datetimeall(const TDSCONTEXT * tds_ctx, int srctype, const TDS_DATET
 		tds_strftime(whole_date_string, sizeof(whole_date_string), tds_ctx->locale->date_fmt, &when, 
 		             dta->time_prec);
 
-		return string_to_result(whole_date_string, cr);
+		return string_to_result(desttype, whole_date_string, cr);
 	case SYBDATETIME:
 		if (!IS_INT(dta->date))
 			return TDS_CONVERT_OVERFLOW;
@@ -1342,7 +1339,7 @@ tds_convert_datetime(const TDSCONTEXT * tds_ctx, const TDS_DATETIME * dt, int de
 		tds_datecrack(SYBDATETIME, dt, &when);
 		tds_strftime(whole_date_string, sizeof(whole_date_string), tds_ctx->locale->date_fmt, &when, 3);
 
-		return string_to_result(whole_date_string, cr);
+		return string_to_result(desttype, whole_date_string, cr);
 	case SYBDATETIME:
 		cr->dt = *dt;
 		return sizeof(TDS_DATETIME);
@@ -1511,7 +1508,7 @@ tds_convert_real(const TDS_REAL* src, int desttype, CONV_RESULT * cr)
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
 		sprintf(tmp_str, "%.9g", the_value);
-		return string_to_result(tmp_str, cr);
+		return string_to_result(desttype, tmp_str, cr);
 		break;
 	case SYBINT1:
 	case SYBUINT1:
@@ -1620,7 +1617,7 @@ tds_convert_flt8(const TDS_FLOAT* src, int desttype, CONV_RESULT * cr)
 	case TDS_CONVERT_CHAR:
 	case CASE_ALL_CHAR:
 		sprintf(tmp_str, "%.17g", the_value);
-		return string_to_result(tmp_str, cr);
+		return string_to_result(desttype, tmp_str, cr);
 		break;
 	case SYBINT1:
 	case SYBUINT1:
@@ -1725,7 +1722,7 @@ tds_convert_unique(const TDS_CHAR * src, int desttype, CONV_RESULT * cr)
 		sprintf(buf, "%08X-%04X-%04X-%02X%02X-%02X%02X%02X%02X%02X%02X",
 			(int) u->Data1, (int) u->Data2, (int) u->Data3,
 			u->Data4[0], u->Data4[1], u->Data4[2], u->Data4[3], u->Data4[4], u->Data4[5], u->Data4[6], u->Data4[7]);
-		return string_to_result(buf, cr);
+		return string_to_result(desttype, buf, cr);
 		break;
 	case SYBUNIQUE:
 		/*
@@ -1735,7 +1732,7 @@ tds_convert_unique(const TDS_CHAR * src, int desttype, CONV_RESULT * cr)
 		memcpy(&(cr->u), src, sizeof(TDS_UNIQUE));
 		return sizeof(TDS_UNIQUE);
 		break;
-		/* no not warning for not convertible types */
+		/* do not warn for not convertible types */
 	case SYBBIT:
 	case SYBBITN:
 	case SYBINT1:
