@@ -196,6 +196,7 @@
 #endif /* HAVE_STDLIB_H */
 
 #define TDS_DONT_DEFINE_DEFAULT_FUNCTIONS
+#include <freetds/utils.h>
 #include <freetds/tds.h>
 #include <freetds/bytes.h>
 #include <freetds/iconv.h>
@@ -217,7 +218,8 @@ static void tds_swap_numeric(TDS_NUMERIC *num);
 #define MAX(a,b) (((a) > (b)) ? (a) : (b))
 
 /**
- * Set type of column initializing all dependency 
+ * Set type of column initializing all dependency.
+ * column_usertype should already be set.
  * @param curcol column to set
  * @param type   type to set
  */
@@ -260,13 +262,22 @@ tds_set_param_type(TDSCONNECTION * conn, TDSCOLUMN * curcol, TDS_SERVER_TYPE typ
 		case SYBBINARY:
 			type = XSYBBINARY;
 			break;
+		case SYBBIT:
+			type = SYBBITN;
+			break;
 			/* avoid warning on other types */
 		default:
 			break;
 		}
 	} else if (IS_TDS50(conn)) {
-		if (type == SYBINT8)
+		switch (type) {
+		case SYBINT8:
 			type = SYB5INT8;
+			break;
+			/* avoid warning on other types */
+		default:
+			break;
+		}
 	}
 	tds_set_column_type(conn, curcol, type);
 
@@ -468,7 +479,7 @@ tds_get_char_dynamic(TDSSOCKET *tds, TDSCOLUMN *curcol, void **pp, size_t alloca
 	if (USE_ICONV && curcol->char_conv)
 		res = tds_convert_stream(tds, curcol->char_conv, to_client, r_stream, &w.stream);
 	else
-		res = tds_copy_stream(tds, r_stream, &w.stream);
+		res = tds_copy_stream(r_stream, &w.stream);
 	curcol->column_cur_size = w.size;
 	return res;
 }
@@ -811,7 +822,7 @@ tds_generic_get(TDSSOCKET * tds, TDSCOLUMN * curcol)
 			discard_len = colsize - curcol->column_size;
 			colsize = curcol->column_size;
 		}
-		if (tds_get_n(tds, dest, colsize) == NULL)
+		if (!tds_get_n(tds, dest, colsize))
 			return TDS_FAIL;
 		if (discard_len > 0)
 			tds_get_n(tds, NULL, discard_len);
@@ -884,8 +895,7 @@ tds_generic_put_info(TDSSOCKET * tds, TDSCOLUMN * col)
 	}
 
 	/* TDS5 wants a table name for LOBs */
-	if (IS_TDS50(tds->conn)
-	    && (col->on_server.column_type == SYBIMAGE || col->on_server.column_type == SYBTEXT))
+	if (IS_TDS50(tds->conn) && is_blob_type(col->on_server.column_type))
 		tds_put_smallint(tds, 0);
 
 	/* TDS7.1 output collate information */
@@ -1429,6 +1439,12 @@ tds_clrudt_row_len(TDSCOLUMN *col)
 	return sizeof(TDSBLOB);
 }
 
+unsigned
+tds_clrudt_put_info_len(TDSSOCKET * tds, TDSCOLUMN * col)
+{
+	return 3;
+}
+
 TDSRET
 tds_clrudt_put_info(TDSSOCKET * tds, TDSCOLUMN * col)
 {
