@@ -38,6 +38,7 @@
 
 #include <freetds/tds.h>
 #include <freetds/iconv.h>
+#include <freetds/bool.h>
 #if HAVE_ICONV
 #include <iconv.h>
 #endif
@@ -324,6 +325,10 @@ tds_iconv_open(TDSCONNECTION * conn, const char *charset, int use_utf16)
 
 	tdsdump_log(TDS_DBG_FUNC, "tds_iconv_open(%p, %s)\n", conn, charset);
 
+	/* TDS 5.0 support only UTF-16 encodings */
+	if (IS_TDS50(conn))
+		use_utf16 = true;
+
 	/* initialize */
 	if (!iconv_initialized) {
 		if ((ret = tds_iconv_init()) > 0) {
@@ -553,11 +558,11 @@ tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 	iconv_t error_cd = invalid;
 
 	char quest_mark[] = "?";	/* best to leave non-const; implementations vary */
-	ICONV_CONST char *pquest_mark = quest_mark;
+	ICONV_CONST char *pquest_mark;
 	size_t lquest_mark;
 	size_t irreversible;
 	size_t one_character;
-	int eilseq_raised = 0;
+	bool eilseq_raised = false;
 	int conv_errno;
 	/* cast away const-ness */
 	TDS_ERRNO_MESSAGE_FLAGS *suppress = (TDS_ERRNO_MESSAGE_FLAGS*) &conv->suppress;
@@ -625,7 +630,7 @@ tds_iconv(TDSSOCKET * tds, TDSICONV * conv, TDS_ICONV_DIRECTION io,
 		conv_errno = errno;
 
 		if (conv_errno == EILSEQ)
-			eilseq_raised = 1;
+			eilseq_raised = true;
 
 		if (conv_errno != EILSEQ || io != to_client || !inbuf)
 			break;
@@ -828,13 +833,8 @@ skip_one_input_sequence(iconv_t cd, const TDS_ENCODING * charset, const char **i
 
 
 	/* usually fixed size and UTF-8 do not have state, so do not reset it */
-	if (charsize) {
-		if (charsize > *input_size)
-			return 0;
-		*input += charsize;
-		*input_size -= charsize;
-		return charsize;
-	}
+	if (charsize)
+		goto skip_charsize;
 
 	if (0 == strcmp(charset->name, "UTF-8")) {
 		/*
@@ -851,11 +851,7 @@ skip_one_input_sequence(iconv_t cd, const TDS_ENCODING * charset, const char **i
 		do {
 			++charsize;
 		} while ((c <<= 1) & 0x80);
-		if (charsize > *input_size)
-			return 0;
-		*input += charsize;
-		*input_size -= charsize;
-		return charsize;
+		goto skip_charsize;
 	}
 
 	/* handle state encoding */
@@ -905,7 +901,20 @@ skip_one_input_sequence(iconv_t cd, const TDS_ENCODING * charset, const char **i
 
 	tds_sys_iconv_close(cd2);
 
-	return l;
+	if (l != 0)
+		return l;
+
+	/* last blindly attempt, skip minimum bytes */
+	charsize = charset->min_bytes_per_char;
+
+	/* fall through */
+
+skip_charsize:
+	if (charsize > *input_size)
+		return 0;
+	*input += charsize;
+	*input_size -= charsize;
+	return charsize;
 }
 
 static int
@@ -1024,7 +1033,7 @@ collate2charset(int sql_collate, int lcid)
 	case 0x41c:
 	case 0x424:
 	case 0x442:
-		/* case 0x81a: seem wrong in XP table TODO check */
+	case 0x81a:
 	case 0x104e:		/* ?? */
 	case 0x141a:
 		cp = TDS_CHARSET_CP1250;
@@ -1038,10 +1047,12 @@ collate2charset(int sql_collate, int lcid)
 	case 0x440:
 	case 0x444:
 	case 0x450:
-	case 0x81a:		/* ?? */
 	case 0x82c:
 	case 0x843:
 	case 0xc1a:
+	case 0x46d:
+	case 0x201a:
+	case 0x485:
 		cp = TDS_CHARSET_CP1251;
 		break;
 	case 0x1007:
@@ -1057,7 +1068,6 @@ collate2charset(int sql_collate, int lcid)
 	case 0x180c:
 	case 0x1c09:
 	case 0x1c0a:
-	case 0x201a:
 	case 0x2009:
 	case 0x200a:
 	case 0x2409:
@@ -1105,8 +1115,6 @@ collate2charset(int sql_collate, int lcid)
 	case 0x440a:
 	case 0x441:
 	case 0x456:
-	case 0x46d:
-	case 0x485:
 	case 0x480a:
 	case 0x4c0a:
 	case 0x500a:
