@@ -32,9 +32,10 @@
 #include <freetds/iconv.h>
 #include <freetds/tls.h>
 #include <freetds/checks.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #include "replacements.h"
 #include <freetds/enum_cap.h>
+#include <freetds/utils.h>
 
 #if HAVE_SYS_SOCKET_H
 #include <sys/socket.h>
@@ -117,7 +118,9 @@ tds_get_dynid(TDSCONNECTION * conn, char *id)
 	return id;
 }
 
+#include <freetds/pushvis.h>
 extern const TDSCOLUMNFUNCS tds_invalid_funcs;
+#include <freetds/popvis.h>
 
 static TDSCOLUMN *
 tds_alloc_column(void)
@@ -658,7 +661,7 @@ tds_free_all_results(TDSSOCKET * tds)
 	tds_free_param_results(tds->param_info);
 	tds->param_info = NULL;
 	tds_free_compute_results(tds);
-	tds->has_status = 0;
+	tds->has_status = false;
 	tds->in_row = false;
 	tds->ret_status = 0;
 	if (tds->cur_dyn)
@@ -721,7 +724,7 @@ tds_alloc_context(void * parent)
 	}
 	context->locale = locale;
 	context->parent = parent;
-	context->money_use_2_digits = true;
+	context->money_use_2_digits = false;
 
 	return context;
 }
@@ -805,10 +808,10 @@ tds_init_login(TDSLOGIN *login, TDSLOCALE * locale)
 
 	/*
 	 * TDS 7.0:
-	 * 0x02 indicates ODBC driver
-	 * 0x01 means change to initial language must succeed
+	 * ODBC driver settings.
+	 * Change to initial language must succeed.
 	 */
-	login->option_flag2 = 0x03;
+	login->option_flag2 = TDS_INIT_LANG_REQUIRED|TDS_ODBC_ON;
 	login->tds_version = TDS_DEFAULT_VERSION;
 	login->block_size = 0;
 
@@ -985,7 +988,6 @@ tds_alloc_login(int use_environment)
 	tds_dstr_init(&login->new_password);
 
 	login->ip_addrs = NULL;
-	login->connected_addr = NULL;
 
 	tds_dstr_init(&login->database);
 	tds_dstr_init(&login->dump_file);
@@ -1014,8 +1016,11 @@ tds_alloc_login(int use_environment)
 	}
 
 	login->capabilities = defaultcaps;
+	login->use_ntlmv2_specified = 0;
+	login->use_ntlmv2 = 1;
+	login->enable_tls_v1 = 1;
 
-	Cleanup:
+Cleanup:
 	return login;
 }
 
