@@ -187,7 +187,7 @@ tds_get_conversion_type(TDS_SERVER_TYPE srctype, int colsize)
 	return srctype;
 }
 
-const unsigned char tds_type_flags_ms[256] = {
+const uint16_t tds_type_flags_ms[256] = {
 	/*   0 empty                */	TDS_TYPEFLAG_INVALID,
 	/*   1 empty                */	TDS_TYPEFLAG_INVALID,
 	/*   2 empty                */	TDS_TYPEFLAG_INVALID,
@@ -222,10 +222,10 @@ const unsigned char tds_type_flags_ms[256] = {
 	/*  31 SYBVOID              */	TDS_TYPEFLAG_FIXED,
 	/*  32 empty                */	TDS_TYPEFLAG_INVALID,
 	/*  33 empty                */	TDS_TYPEFLAG_INVALID,
-	/*  34 SYBIMAGE             */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/*  34 SYBIMAGE             */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/*  35 SYBTEXT              */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_COLLATE|TDS_TYPEFLAG_ASCII,
 	/*  36 SYBUNIQUE            */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_FIXED,
-	/*  37 SYBVARBINARY         */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/*  37 SYBVARBINARY         */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/*  38 SYBINTN              */	TDS_TYPEFLAG_NULLABLE,
 	/*  39 SYBVARCHAR           */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_ASCII,
 	/*  40 SYBMSDATE            */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_FIXED|TDS_TYPEFLAG_DATETIME,
@@ -233,7 +233,7 @@ const unsigned char tds_type_flags_ms[256] = {
 	/*  42 SYBMSDATETIME2       */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_DATETIME,
 	/*  43 SYBMSDATETIMEOFFSET  */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_DATETIME,
 	/*  44 empty                */	TDS_TYPEFLAG_INVALID,
-	/*  45 SYBBINARY            */	TDS_TYPEFLAG_VARIABLE,
+	/*  45 SYBBINARY            */	TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/*  46 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_FIXED,
 	/*  47 SYBCHAR              */	TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_ASCII,
 	/*  48 SYBINT1              */	TDS_TYPEFLAG_FIXED,
@@ -353,7 +353,7 @@ const unsigned char tds_type_flags_ms[256] = {
 	/* 162 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 163 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
 	/* 164 empty                */	TDS_TYPEFLAG_INVALID,
-	/* 165 XSYBVARBINARY        */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/* 165 XSYBVARBINARY        */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/* 166 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 167 XSYBVARCHAR          */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_COLLATE|TDS_TYPEFLAG_ASCII,
 	/* 168 empty                */	TDS_TYPEFLAG_INVALID,
@@ -361,7 +361,7 @@ const unsigned char tds_type_flags_ms[256] = {
 	/* 170 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 171 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 172 empty                */	TDS_TYPEFLAG_INVALID,
-	/* 173 XSYBBINARY           */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/* 173 XSYBBINARY           */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/* 174 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_UNICODE,
 	/* 175 XSYBCHAR             */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_COLLATE|TDS_TYPEFLAG_ASCII,
 	/* 176 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_FIXED,
@@ -413,7 +413,7 @@ const unsigned char tds_type_flags_ms[256] = {
 	/* 222 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 223 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 224 empty                */	TDS_TYPEFLAG_INVALID,
-	/* 225 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/* 225 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/* 226 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 227 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 228 empty                */	TDS_TYPEFLAG_INVALID,
@@ -447,7 +447,7 @@ const unsigned char tds_type_flags_ms[256] = {
 };
 
 #if 0
-const unsigned char tds_type_flags_syb[256] = {
+const uint16_t tds_type_flags_syb[256] = {
 	/*   0 empty                */	TDS_TYPEFLAG_INVALID,
 	/*   1 empty                */	TDS_TYPEFLAG_INVALID,
 	/*   2 empty                */	TDS_TYPEFLAG_INVALID,
@@ -482,10 +482,10 @@ const unsigned char tds_type_flags_syb[256] = {
 	/*  31 SYBVOID              */	TDS_TYPEFLAG_FIXED,
 	/*  32 empty                */	TDS_TYPEFLAG_INVALID,
 	/*  33 empty                */	TDS_TYPEFLAG_INVALID,
-	/*  34 SYBIMAGE             */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/*  34 SYBIMAGE             */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/*  35 SYBTEXT              */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_COLLATE|TDS_TYPEFLAG_ASCII,
 	/*  36 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_FIXED|TDS_TYPEFLAG_NULLABLE,
-	/*  37 SYBVARBINARY         */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/*  37 SYBVARBINARY         */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/*  38 SYBINTN              */	TDS_TYPEFLAG_NULLABLE,
 	/*  39 SYBVARCHAR           */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_ASCII,
 	/*  40 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_FIXED|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_DATETIME,
@@ -493,7 +493,7 @@ const unsigned char tds_type_flags_syb[256] = {
 	/*  42 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_DATETIME,
 	/*  43 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_DATETIME,
 	/*  44 empty                */	TDS_TYPEFLAG_INVALID,
-	/*  45 SYBBINARY            */	TDS_TYPEFLAG_VARIABLE,
+	/*  45 SYBBINARY            */	TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/*  46 SYBINTERVAL          */	TDS_TYPEFLAG_FIXED,
 	/*  47 SYBCHAR              */	TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_ASCII,
 	/*  48 SYBINT1              */	TDS_TYPEFLAG_FIXED,
@@ -613,7 +613,7 @@ const unsigned char tds_type_flags_syb[256] = {
 	/* 162 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 163 SYBXML               */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
 	/* 164 empty                */	TDS_TYPEFLAG_INVALID,
-	/* 165 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/* 165 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/* 166 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 167 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
 	/* 168 empty                */	TDS_TYPEFLAG_INVALID,
@@ -621,7 +621,7 @@ const unsigned char tds_type_flags_syb[256] = {
 	/* 170 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 171 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 172 empty                */	TDS_TYPEFLAG_INVALID,
-	/* 173 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/* 173 empty                */	TDS_TYPEFLAG_INVALID|TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/* 174 SYBUNITEXT           */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_UNICODE,
 	/* 175 SYBLONGCHAR          */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_ASCII,
 	/* 176 SYBSINT1             */	TDS_TYPEFLAG_FIXED,
@@ -673,7 +673,7 @@ const unsigned char tds_type_flags_syb[256] = {
 	/* 222 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 223 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 224 empty                */	TDS_TYPEFLAG_INVALID,
-	/* 225 SYBLONGBINARY        */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE,
+	/* 225 SYBLONGBINARY        */	TDS_TYPEFLAG_NULLABLE|TDS_TYPEFLAG_VARIABLE|TDS_TYPEFLAG_BINARY,
 	/* 226 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 227 empty                */	TDS_TYPEFLAG_INVALID,
 	/* 228 empty                */	TDS_TYPEFLAG_INVALID,
