@@ -20,6 +20,9 @@
 
 #include <config.h>
 
+/* enabled some additional definitions for inet_pton */
+#define _WIN32_WINNT 0x600
+
 #include <stdio.h>
 
 #if HAVE_ERRNO_H
@@ -51,7 +54,7 @@
 #endif
 
 #include <freetds/tds.h>
-#include <freetds/string.h>
+#include <freetds/utils/string.h>
 #include <freetds/tls.h>
 #include <freetds/alloca.h>
 #include "replacements.h"
@@ -510,7 +513,10 @@ tds_ssl_init(TDSSOCKET *tds)
 	gnutls_set_default_priority(session);
 
 	/* ... but overwrite some */
-	ret = gnutls_priority_set_direct (session, "NORMAL:%COMPAT:-VERS-SSL3.0", NULL);
+	if (tds->login && tds->login->enable_tls_v1)
+		ret = gnutls_priority_set_direct(session, "NORMAL:%COMPAT:-VERS-SSL3.0", NULL);
+	else
+		ret = gnutls_priority_set_direct(session, "NORMAL:%COMPAT:-VERS-SSL3.0:-VERS-TLS1.0", NULL);
 	if (ret != 0)
 		goto cleanup;
 
@@ -576,12 +582,8 @@ tds_ssl_deinit(TDSCONNECTION *conn)
 static long
 tds_ssl_ctrl_login(BIO *b, int cmd, long num, void *ptr)
 {
-	TDSSOCKET *tds = (TDSSOCKET *) BIO_get_data(b);
-
 	switch (cmd) {
 	case BIO_CTRL_FLUSH:
-		if (tds->out_pos > 8)
-			tds_flush_packet(tds);
 		return 1;
 	}
 	return 0;
@@ -658,6 +660,71 @@ tds_deinit_openssl_methods(void)
 #  endif
 #endif
 
+#if OPENSSL_VERSION_NUMBER < 0x1010000FL || defined(LIBRESSL_VERSION_NUMBER)
+static tds_mutex *openssl_locks;
+
+static void
+openssl_locking_callback(int mode, int type, const char *file, int line)
+{
+	if (mode & CRYPTO_LOCK)
+		tds_mutex_lock(&openssl_locks[type]);
+	else
+		tds_mutex_unlock(&openssl_locks[type]);
+}
+
+static void
+tds_init_openssl_thread(void)
+{
+	int i, n = CRYPTO_num_locks();
+
+	/* if already set do not overwrite,
+	 * application or another library took care of */
+	if (CRYPTO_get_locking_callback())
+		return;
+
+	openssl_locks = tds_new(tds_mutex, n);
+	for (i=0; i < n; ++i)
+		tds_mutex_init(&openssl_locks[i]);
+
+	/* read back in the attempt to avoid race conditions
+	 * this is not safe but there are no race free ways */
+	if (CRYPTO_get_locking_callback() == NULL)
+		CRYPTO_set_locking_callback(openssl_locking_callback);
+	if (CRYPTO_get_locking_callback() == openssl_locking_callback)
+		return;
+
+	for (i=0; i < n; ++i)
+		tds_mutex_free(&openssl_locks[i]);
+	free(openssl_locks);
+	openssl_locks = NULL;
+}
+
+#ifdef TDS_ATTRIBUTE_DESTRUCTOR
+static void __attribute__((destructor))
+tds_deinit_openssl(void)
+{
+	int i, n;
+
+	if (!tls_initialized
+	    || CRYPTO_get_locking_callback() != openssl_locking_callback)
+		return;
+
+	CRYPTO_set_locking_callback(NULL);
+	n = CRYPTO_num_locks();
+	for (i=0; i < n; ++i)
+		tds_mutex_free(&openssl_locks[i]);
+	free(openssl_locks);
+	openssl_locks = NULL;
+}
+#endif
+
+#else
+static inline void
+tds_init_openssl_thread(void)
+{
+}
+#endif
+
 static SSL_CTX *
 tds_init_openssl(void)
 {
@@ -667,6 +734,7 @@ tds_init_openssl(void)
 		tds_mutex_lock(&tls_mutex);
 		if (!tls_initialized) {
 			SSL_library_init();
+			tds_init_openssl_thread();
 			tds_init_ssl_methods();
 			tls_initialized = 1;
 		}
@@ -863,16 +931,18 @@ check_hostname(X509 *cert, const char *hostname)
 int
 tds_ssl_init(TDSSOCKET *tds)
 {
-#define DEFAULT_OPENSSL_CTX_OPTIONS (SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3)
+#define DEFAULT_OPENSSL_CTX_OPTIONS (SSL_OP_NO_SSLv2 | SSL_OP_NO_SSLv3 | SSL_OP_NO_TLSv1)
 #define DEFAULT_OPENSSL_CIPHERS "HIGH:!SSLv2:!aNULL:-DH"
 
 	SSL *con;
 	SSL_CTX *ctx;
 	BIO *b, *b2;
 
-	int ret;
+	int ret, connect_ret;
 	const char *tls_msg;
 
+	unsigned long ctx_options = DEFAULT_OPENSSL_CTX_OPTIONS;
+
 	con = NULL;
 	b = NULL;
 	b2 = NULL;
@@ -887,7 +957,9 @@ tds_ssl_init(TDSSOCKET *tds)
 	if (!ctx)
 		goto cleanup;
 
-	SSL_CTX_set_options(ctx, DEFAULT_OPENSSL_CTX_OPTIONS);
+	if (tds->login && tds->login->enable_tls_v1)
+		ctx_options &= ~SSL_OP_NO_TLSv1;
+	SSL_CTX_set_options(ctx, ctx_options);
 
 	if (!tds_dstr_isempty(&tds->login->cafile)) {
 		tls_msg = "loading CA file";
@@ -948,10 +1020,19 @@ tds_ssl_init(TDSSOCKET *tds)
 
 	/* Perform the TLS handshake */
 	tls_msg = "handshake";
+	ERR_clear_error();
 	SSL_set_connect_state(con);
-	ret = SSL_connect(con) != 1 || SSL_get_state(con) != TLS_ST_OK;
-	if (ret != 0)
+	connect_ret = SSL_connect(con);
+	ret = connect_ret != 1 || SSL_get_state(con) != TLS_ST_OK;
+	if (ret != 0) {
+		tdsdump_log(TDS_DBG_ERROR, "handshake failed with %d %d %d\n",
+			    connect_ret, SSL_get_state(con), SSL_get_error(con, connect_ret));
 		goto cleanup;
+	}
+
+	/* flush pending data */
+	if (tds->out_pos > 8)
+		tds_flush_packet(tds);
 
 	/* check certificate hostname */
 	if (!tds_dstr_isempty(&tds->login->cafile) && tds->login->check_ssl_hostname) {
@@ -994,11 +1075,11 @@ tds_ssl_deinit(TDSCONNECTION *conn)
 {
 	if (conn->tls_session) {
 		/* NOTE do not call SSL_shutdown here */
-		SSL_free(conn->tls_session);
+		SSL_free((SSL *) conn->tls_session);
 		conn->tls_session = NULL;
 	}
 	if (conn->tls_ctx) {
-		SSL_CTX_free(conn->tls_ctx);
+		SSL_CTX_free((SSL_CTX *) conn->tls_ctx);
 		conn->tls_ctx = NULL;
 	}
 }
