@@ -1,12 +1,8 @@
 #ifndef COMMON_h
 #define COMMON_h
 
-static char rcsid_common_h[] = "$Id: common.h,v 1.9 2010/12/30 12:04:52 freddy77 Exp $";
-static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_warn };
-
-#if HAVE_CONFIG_H
+#undef NDEBUG
 #include <config.h>
-#endif /* HAVE_CONFIG_H */
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -19,7 +15,12 @@ static void *no_unused_common_h_warn[] = { rcsid_common_h, no_unused_common_h_wa
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include <tds.h>
+#include <freetds/tds.h>
+#include <freetds/data.h>
+
+#ifndef FREETDS_SRCDIR
+#define FREETDS_SRCDIR FREETDS_TOPDIR "/src/tds/unittests"
+#endif
 
 extern char PASSWORD[512];
 extern char USER[512];
@@ -39,4 +40,7 @@ extern int utf8_max_len;
 int get_unichar(const char **psrc);
 char *to_utf8(const char *src, char *dest);
 
+typedef void tds_any_type_t(TDSSOCKET *tds, TDSCOLUMN *col);
+void tds_all_types(TDSSOCKET *tds, tds_any_type_t *func);
+
 #endif
