@@ -18,9 +18,6 @@
  */
 #include "common.h"
 
-static char software_version[] = "$Id: dynamic1.c,v 1.19 2009/01/16 20:27:59 jklowden Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
 static int discard_result(TDSSOCKET * tds);
 
 static void
@@ -44,15 +41,11 @@ test(TDSSOCKET * tds, TDSDYNAMIC * dyn, TDS_INT n, const char *s)
 	dyn->params = params;
 
 	curcol = params->columns[0];
-	curcol->column_type = SYBINTN;
-	curcol->on_server.column_type = SYBINTN;
-	curcol->column_size = sizeof(TDS_INT);
-	curcol->on_server.column_size = sizeof(TDS_INT);
-	curcol->column_varint_size = 1;
-	curcol->column_cur_size = sizeof(TDS_INT);
+	tds_set_param_type(tds->conn, curcol, SYBINT4);
 
 	/* TODO test error */
 	tds_alloc_param_data(curcol);
+	curcol->column_cur_size = sizeof(TDS_INT);
 	memcpy(curcol->column_data, &n, sizeof(n));
 
 	if (!(params = tds_alloc_param_result(dyn->params)))
@@ -60,16 +53,16 @@ test(TDSSOCKET * tds, TDSDYNAMIC * dyn, TDS_INT n, const char *s)
 	dyn->params = params;
 
 	curcol = params->columns[1];
-	tds_set_param_type(tds, curcol, SYBVARCHAR);
+	tds_set_param_type(tds->conn, curcol, SYBVARCHAR);
 	curcol->column_size = 40;
 	curcol->column_cur_size = len;
 
 	tds_alloc_param_data(curcol);
 	memcpy(curcol->column_data, s, len);
 
-	if (tds_submit_execute(tds, dyn) != TDS_SUCCEED)
+	if (tds_submit_execute(tds, dyn) != TDS_SUCCESS)
 		fatal_error("tds_submit_execute() error");
-	if (discard_result(tds) != TDS_SUCCEED)
+	if (discard_result(tds) != TDS_SUCCESS)
 		fatal_error("tds_submit_execute() output error");
 }
 
@@ -84,20 +77,20 @@ main(int argc, char **argv)
 
 	fprintf(stdout, "%s: Test dynamic queries\n", __FILE__);
 	rc = try_tds_login(&login, &tds, __FILE__, verbose);
-	if (rc != TDS_SUCCEED)
+	if (rc != TDS_SUCCESS)
 		fatal_error("try_tds_login() failed");
 
 	run_query(tds, "DROP TABLE #dynamic1");
-	if (run_query(tds, "CREATE TABLE #dynamic1 (i INT, c VARCHAR(40))") != TDS_SUCCEED)
+	if (run_query(tds, "CREATE TABLE #dynamic1 (i INT, c VARCHAR(40))") != TDS_SUCCESS)
 		fatal_error("creating table error");
 
 	if (tds->cur_dyn)
 		fatal_error("already a dynamic query??");
 
 	/* prepare to insert */
-	if (tds_submit_prepare(tds, "INSERT INTO #dynamic1(i,c) VALUES(?,?)", NULL, &dyn, NULL) != TDS_SUCCEED)
+	if (tds_submit_prepare(tds, "INSERT INTO #dynamic1(i,c) VALUES(?,?)", NULL, &dyn, NULL) != TDS_SUCCESS)
 		fatal_error("tds_submit_prepare() error");
-	if (discard_result(tds) != TDS_SUCCEED)
+	if (discard_result(tds) != TDS_SUCCESS)
 		fatal_error("tds_submit_prepare() output error");
 
 	if (!dyn)
@@ -107,33 +100,35 @@ main(int argc, char **argv)
 	test(tds, dyn, 123, "dynamic");
 
 	/* some test */
-	if (run_query(tds, "DECLARE @n INT SELECT @n = COUNT(*) FROM #dynamic1 IF @n <> 1 SELECT 0") != TDS_SUCCEED)
+	if (run_query(tds, "DECLARE @n INT SELECT @n = COUNT(*) FROM #dynamic1 IF @n <> 1 SELECT 0") != TDS_SUCCESS)
 		fatal_error("checking rows");
 
 	if (run_query(tds, "DECLARE @n INT SELECT @n = COUNT(*) FROM #dynamic1 WHERE i = 123 AND c = 'dynamic' IF @n <> 1 SELECT 0")
-	    != TDS_SUCCEED)
+	    != TDS_SUCCESS)
 		fatal_error("checking rows 1");
 
 	/* insert one record */
 	test(tds, dyn, 654321, "a longer string");
 
 	/* some test */
-	if (run_query(tds, "DECLARE @n INT SELECT @n = COUNT(*) FROM #dynamic1 IF @n <> 2 SELECT 0") != TDS_SUCCEED)
+	if (run_query(tds, "DECLARE @n INT SELECT @n = COUNT(*) FROM #dynamic1 IF @n <> 2 SELECT 0") != TDS_SUCCESS)
 		fatal_error("checking rows");
 
 	if (run_query(tds, "DECLARE @n INT SELECT @n = COUNT(*) FROM #dynamic1 WHERE i = 123 AND c = 'dynamic' IF @n <> 1 SELECT 0")
-	    != TDS_SUCCEED)
+	    != TDS_SUCCESS)
 		fatal_error("checking rows 2");
 
 	if (run_query
 	    (tds,
 	     "DECLARE @n INT SELECT @n = COUNT(*) FROM #dynamic1 WHERE i = 654321 AND c = 'a longer string' IF @n <> 1 SELECT 0") !=
-	    TDS_SUCCEED)
+	    TDS_SUCCESS)
 		fatal_error("checking rows 3");
 
-	if (run_query(tds, "DROP TABLE #dynamic1") != TDS_SUCCEED)
+	if (run_query(tds, "DROP TABLE #dynamic1") != TDS_SUCCESS)
 		fatal_error("dropping table error");
 
+	tds_release_dynamic(&dyn);
+
 	try_tds_logout(login, tds, verbose);
 	return 0;
 }
@@ -144,7 +139,7 @@ discard_result(TDSSOCKET * tds)
 	int rc;
 	int result_type;
 
-	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCEED) {
+	while ((rc = tds_process_tokens(tds, &result_type, NULL, TDS_TOKEN_RESULTS)) == TDS_SUCCESS) {
 
 		switch (result_type) {
 		case TDS_DONE_RESULT:
@@ -167,5 +162,5 @@ discard_result(TDSSOCKET * tds)
 		return TDS_FAIL;
 	}
 
-	return TDS_SUCCEED;
+	return TDS_SUCCESS;
 }
