@@ -18,10 +18,6 @@
  */
 #include "common.h"
 
-static char software_version[] = "$Id: t0003.c,v 1.17 2004/12/02 13:20:45 freddy77 Exp $";
-static void *no_unused_var_warn[] = { software_version, no_unused_var_warn };
-
-
 int
 main(int argc, char **argv)
 {
@@ -32,34 +28,34 @@ main(int argc, char **argv)
 
 	fprintf(stdout, "%s: Testing DB change -- 'use tempdb'\n", __FILE__);
 	rc = try_tds_login(&login, &tds, __FILE__, verbose);
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		fprintf(stderr, "try_tds_login() failed\n");
 		return 1;
 	}
 
 	rc = tds_submit_query(tds, "use tempdb");
-	if (rc != TDS_SUCCEED) {
+	if (rc != TDS_SUCCESS) {
 		fprintf(stderr, "tds_submit_query() failed\n");
 		return 1;
 	}
 
 	/* warning: this mucks with some internals to get the env chg message */
-	if (tds_process_simple_query(tds) != TDS_SUCCEED) {
+	if (tds_process_simple_query(tds) != TDS_SUCCESS) {
 		fprintf(stderr, "query results failed\n");
 		return 1;
 	}
 
-	if (!tds || !tds->env.database) {
+	if (!tds || !tds->conn->env.database) {
 		fprintf(stderr, "No database ??\n");
 		return 1;
 	}
 
 	/* Test currently disabled during TDSENV changes */
 	if (verbose) {
-		fprintf(stdout, "database changed to %s\n", tds->env.database);
+		fprintf(stdout, "database changed to %s\n", tds->conn->env.database);
 	}
-	if (strcmp(tds->env.database, "tempdb")) {
-		fprintf(stderr, "Wrong database, %s != tempdb\n", tds->env.database);
+	if (strcmp(tds->conn->env.database, "tempdb")) {
+		fprintf(stderr, "Wrong database, %s != tempdb\n", tds->conn->env.database);
 		return 1;
 	}
 
