@@ -17,9 +17,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#if HAVE_CONFIG_H
 #include <config.h>
-#endif
 
 #include <stdarg.h>
 #include <stdio.h>
@@ -32,11 +30,9 @@
 #include <string.h>
 #endif /* HAVE_STRING_H */
 
-#include "tds.h"
+#include <freetds/tds.h>
 #include "replacements.h"
 
-TDS_RCSID(var, "$Id: vstrbuild.c,v 1.16 2009/01/16 20:27:59 jklowden Exp $");
-
 struct string_linked_list
 {
 	char *str;
@@ -49,7 +45,7 @@ struct string_linked_list
  */
 
 static char *
-norm_fmt(const char *fmt, size_t fmtlen)
+norm_fmt(const char *fmt, int fmtlen)
 {
 	char *newfmt;
 	char *cp;
@@ -58,9 +54,9 @@ norm_fmt(const char *fmt, size_t fmtlen)
 	if (fmtlen == TDS_NULLTERM) {
 		fmtlen = strlen(fmt);
 	}
-	if ((newfmt = (char *) malloc(fmtlen + 1)) == NULL) {
-		return TDS_FAIL;
-	}
+	if ((newfmt = tds_new(char, fmtlen + 1)) == NULL)
+		return NULL;
+
 	for (cp = newfmt; fmtlen > 0; fmtlen--, fmt++) {
 		switch (*fmt) {
 		case ',':
@@ -80,8 +76,8 @@ norm_fmt(const char *fmt, size_t fmtlen)
 	return newfmt;
 }
 
-int
-tds_vstrbuild(char *buffer, int buflen, int *resultlen, char *text, int textlen, const char *formats, int formatlen, va_list ap)
+TDSRET
+tds_vstrbuild(char *buffer, int buflen, int *resultlen, const char *text, int textlen, const char *formats, int formatlen, va_list ap)
 {
 	char *newformat;
 	char *params;
@@ -98,7 +94,7 @@ tds_vstrbuild(char *buffer, int buflen, int *resultlen, char *text, int textlen,
 	int pnum = 0;
 	int pdigit;
 	char *paramp = NULL;
-	int rc = TDS_FAIL;
+	TDSRET rc = TDS_FAIL;
 
 	*resultlen = 0;
 	if (textlen == TDS_NULLTERM) {
@@ -113,7 +109,7 @@ tds_vstrbuild(char *buffer, int buflen, int *resultlen, char *text, int textlen,
 	}
 	free(newformat);
 	for (token = strtok_r(params, sep, &lasts); token != NULL; token = strtok_r(NULL, sep, &lasts)) {
-		if ((*tail = (struct string_linked_list *) malloc(sizeof(struct string_linked_list))) == NULL) {
+		if ((*tail = tds_new(struct string_linked_list, 1)) == NULL) {
 			goto out;
 		}
 		(*tail)->str = token;
@@ -121,7 +117,7 @@ tds_vstrbuild(char *buffer, int buflen, int *resultlen, char *text, int textlen,
 		tail = &((*tail)->next);
 		tokcount++;
 	}
-	if ((string_array = (char **) malloc((tokcount + 1) * sizeof(char *))) == NULL) {
+	if ((string_array = tds_new(char *, tokcount + 1)) == NULL) {
 		goto out;
 	}
 	for (item = head, i = 0; i < tokcount; item = item->next, i++) {
@@ -196,7 +192,7 @@ tds_vstrbuild(char *buffer, int buflen, int *resultlen, char *text, int textlen,
 		}
 	}
 
-	rc = TDS_SUCCEED;
+	rc = TDS_SUCCESS;
 
       out:
 	free(string_array);
